@@ -1,11 +1,12 @@
 /*
-   DRAT.C
-   High level interface routines for Drat (interpreter independent).
-
-   $Id: drat.c,v 1.1 1993/09/02 22:48:38 munro Exp $
+ * $Id: drat.c,v 1.1.1.1 2005/09/18 22:04:54 dhmunro Exp $
+ * High level interface routines for Drat (interpreter independent).
+ */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
  */
-/*    Copyright (c) 1994.  The Regents of the University of California.
-                    All rights reserved.  */
 
 #include "drat.h"
 #include "pstdlib.h"
@@ -128,6 +129,7 @@ void IntegFlat(double *opac, double *source,
   path.maxcuts= path.ncuts= 0;
   path.zone= path.pt1= path.pt2= 0;
   path.ds= 0;
+  path.f= 0;
 
   for (j=0 ; j<nrays ; j++) {
     /* first, track the current ray */
@@ -157,6 +159,7 @@ void IntegLinear(double *opac, double *source,
   path.maxcuts= path.ncuts= 0;
   path.zone= path.pt1= path.pt2= 0;
   path.ds= 0;
+  path.f= 0;
 
   for (j=0 ; j<nrays ; j++) {
     /* first, track the current ray */
