@@ -1,18 +1,14 @@
 /*
-   CFFTI.C
-   Swarztrauber complex FFT routines (initialization).
-
-   $Id$
+ * $Id: cffti.c,v 1.1.1.1 2005/09/18 22:04:52 dhmunro Exp $
+ * Swarztrauber complex FFT routines (initialization).
+ */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
  */
 
-
-
-/*-----prototypes of functions defined here-----*/
-extern void cffti1 (long n,double wa[],long ifac[]);
-extern void cffti (long n,double wsave[]);
-/*-----end of prototypes-----*/
-
-
+#include "cfft.h"
 
 /*-----Fortran intrinsics converted-----*/
 extern double sin(double);
