@@ -1,13 +1,12 @@
 /*
- * CGMIN.C
- *
- * $Id: cgmin.c,v 1.1 1993/08/27 17:19:10 munro Exp $
- *
+ * $Id: cgmin.c,v 1.1.1.1 2005/09/18 22:04:36 dhmunro Exp $
  * Define the CGM reader/echoer for GIST.
- *
  */
-/*    Copyright (c) 1994.  The Regents of the University of California.
-                    All rights reserved.  */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
+ */
 
 #include "play.h"
 #include "cgmin.h"
@@ -646,10 +645,10 @@ n_fread(void *ptr, unsigned long nbytes, CGM *cgm)
       }
       n = valid - n;
       if (n > 0) {
-        if (n >= nbytes) {
+        if ((unsigned long)n >= nbytes) {
           memcpy(out, buf, nbytes);
           cgm->pos += nbytes;
-          if (n == nbytes) cgm->valid = 0;
+          if ((unsigned long)n == nbytes) cgm->valid = 0;
           return nread+nbytes;
         }
         memcpy(out, buf, n);
@@ -970,7 +969,7 @@ static long ReadParameters(long nOctets, GpColor *colors, long maxColors)
         currentCmd+= n;
         cmdLength-= n;
       } else {
-        if (n_fread(newParams+nTot, n, currentCGM)!=n) {
+        if (n_fread(newParams+nTot, n, currentCGM)!=(unsigned long)n) {
           Warning("fread failed on CGM file ", currentCGM->name);
           return -1L;
         }
