@@ -1,13 +1,12 @@
 /*
- * HLEVEL.C
- *
- * $Id: hlevel.c,v 1.1 1993/08/27 17:08:43 munro Exp $
- *
+ * $Id: hlevel.c,v 1.2 2007/12/28 20:20:18 thiebaut Exp $
  * Define routines for recommended GIST interactive interface
- *
  */
-/*    Copyright (c) 1994.  The Regents of the University of California.
-                    All rights reserved.  */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
+ */
 
 #include "hlevel.h"
 #include "pstdlib.h"
@@ -74,9 +73,13 @@ void (*g_on_idle)(void) = 0;
 
 void GhBeforeWait(void)
 {
+#ifndef NO_XLIB
   extern void (*g_pending_task)(void);  /* auto disconnect, see xbasic.c */
   if (g_on_idle) g_on_idle();
   if (g_pending_task) g_pending_task();
+#else
+  if (g_on_idle) g_on_idle();
+#endif
   if (currentDevice<0 || !ghDevices[currentDevice].display ||
       animateOn) return;  /* nothing happens in animate mode until
                            * explicit call to GhFMA */
@@ -166,10 +169,10 @@ void GhFMAMode(int hcp, int animate)
         aport.xmax= 2.0;
         aport.ymin= 0.0;
         aport.ymax= 2.0;
-        if (!port) {
+        /* if (!port) { just animate everything */
           port= &aport;
           animateOn= 2;
-        }
+          /* } */
         if (GxAnimate(display, port)) animateOn= 0;
       } else {
         GxDirect(display);
@@ -203,6 +206,39 @@ int GhGetPlotter(void)
   return currentDevice;
 }
 
+/* Query mouse system and position. */
+int
+GhGetMouse(int *sys, double *x, double *y)
+{
+#ifdef NO_XLIB
+  if (sys) *sys = -1;
+  if (x) *x = 0.0;
+  if (y) *y = 0.0;
+  return -1;  
+#else
+  int j, win = -1;
+  if (gxCurrentEngine != NULL) {
+    for (j = 0; j < GH_NDEVS; ++j) {
+      if (ghDevices[j].display == gxCurrentEngine) {
+	win = j;
+	break;
+      }
+    }
+  }
+  if (win >= 0) {
+    if (sys) *sys = gxCurrentSys;
+    if (x) *x = gxCurrentX;
+    if (y) *y = gxCurrentY;
+  } else {
+    if (sys) *sys = -1;
+    if (x) *x = 0.0;
+    if (y) *y = 0.0;
+  }
+  return win;
+#endif
+}
+
+
 #ifndef NO_XLIB
 /* xbasic.c supplies a hook in its error handlers to allow the hlevel
    to clear its display devices */
@@ -376,7 +412,7 @@ int GhGetPalette(int n, GpColorCell **palette)
 {
   *palette= 0;
   if (n==-1) n = currentDevice;
-  else if (n<0 || n>=GH_NDEVS) return 0;
+  if (n<0 || n>=GH_NDEVS) return 0;
   if (ghDevices[n].display)
     return GpGetPalette(ghDevices[n].display, palette);
   else if (ghDevices[n].hcp)
