@@ -1,13 +1,12 @@
 /*
- * TICK.C
- *
- * $Id: tick.c,v 1.1 1993/08/27 17:08:43 munro Exp $
- *
+ * $Id: tick.c,v 1.2 2010/04/01 03:36:20 dhmunro Exp $
  * Implement ticks and labels for GIST coordinate systems
- *
  */
-/*    Copyright (c) 1994.  The Regents of the University of California.
-                    All rights reserved.  */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
+ */
 
 /* NOTE-- probably still a few bugs, especially with log scales.
    I wish the log scale coding were cleaner.  The only bug I've
@@ -1229,8 +1228,8 @@ int GaAltTick(GaTickStyle *ticks, int xIsLog, int yIsLog,
   }
 
   if (ticks->vert.flags & TICK_ANY) {
-    altticks= (ticks->horiz.flags & ALT_TICK)? ytick : 0;
-    altlabel= (ticks->horiz.flags & ALT_LABEL)? ylabel : 0;
+    altticks= (ticks->vert.flags & ALT_TICK)? ytick : 0;
+    altlabel= (ticks->vert.flags & ALT_LABEL)? ylabel : 0;
     FindTicks(wymin, wymax, &ticks->vert, yIsLog);
 
     if (ticks->vert.flags & TICK_C) {
