@@ -1,9 +1,13 @@
 /*
- * tools.c
- * $Id$
+ * $Id: tools.c,v 1.2 2006/07/20 04:28:30 dhmunro Exp $
  * toolkit for computing ray intersections with tets,
  * performing entry point calculations
  */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
+ */
 
 #include "tools.h"
 
@@ -30,57 +34,88 @@
 
 static int bigger_tri(real xy[][3], int tet[], int i0, int i1);
 
-int tet_traverse(real xy[][3], int tet[])
+int
+tet_traverse(real xy[][3], int tet[])
 {
   int i, tet3= tet[3];    /* base is ABC, apex is D, compute AxD */
-  real cross= xy[tet[0]][0]*xy[tet3][1] - xy[tet[0]][1]*xy[tet3][0];
+  real xd = xy[tet3][0];
+  real yd = xy[tet3][1];
+  real xa = xy[tet[0]][0];
+  real ya = xy[tet[0]][1];
+  real xb = xy[tet[1]][0];
+  real yb = xy[tet[1]][1];
+  real xc = xy[tet[2]][0];
+  real yc = xy[tet[2]][1];
 
-  if         (cross<0.) { /* ray to left of line AD, compute BxD */
-    cross= xy[tet[1]][0]*xy[tet3][1] - xy[tet[1]][1]*xy[tet3][0];
-    if (cross) i= (cross>0.)<<1;
-    else       i= bigger_tri(xy, tet, 2, 0);
+  /* handle degenerate hexahedron cases to avoid roundoff problems */
+  if (xa==xd && ya==yd) {
+    i = 0;
+  } else if (xb==xd && yb==yd) {
+    i = 1;
+  } else if (xc==xd && yc==yd) {
+    i = 2;
 
-  } else if (cross>0.) { /* ray to right of line AD, compute CxD */
-    cross= xy[tet[2]][0]*xy[tet3][1] - xy[tet[2]][1]*xy[tet3][0];
-    if (cross) i= (cross<0.);
-    else       i= bigger_tri(xy, tet, 0, 1);
+  } else {
+    /* stepping into a zero projected area triangle is fatal (tri_intersect)
+     * bigger_tri will never do that, but the other branches here may
+     * - above branches prevent most common case, but do not cover
+     *   case that D is collinear with AB, BC, or CA
+     * definitive check is too expensive to put here?
+     */
+    real cross = xa*yd - ya*xd;
 
-  } else {           /* ray lies exactly on line AD, compute BxD */
-    cross= xy[tet[1]][0]*xy[tet3][1] - xy[tet[1]][1]*xy[tet3][0];
-    if      (cross<0.) i= 0;
-    else if (cross>0.) i= bigger_tri(xy, tet, 1, 2);
-    else               i= bigger_tri(xy, tet, 0, 0); /* D on ray */
+    if         (cross<0.) { /* ray to left of line AD, compute BxD */
+      cross = xb*yd - yb*xd;
+      if (cross) i = (cross>0.)<<1;
+      else       i = bigger_tri(xy, tet, 2, 0);
+
+    } else if (cross>0.) { /* ray to right of line AD, compute CxD */
+      cross = xc*yd - yc*xd;
+      if (cross) i = (cross<0.);
+      else       i = bigger_tri(xy, tet, 0, 1);
+
+    } else {           /* ray lies exactly on line AD, compute BxD */
+      cross = xb*yd - yb*xd;
+      if      (cross<0.) i = 0;
+      else if (cross>0.) i = bigger_tri(xy, tet, 1, 2);
+      else               i = bigger_tri(xy, tet, 0, 0); /* D on ray */
+    }
   }
 
-  tet[3]= tet[i];
-  tet[i]= tet3;
+  tet[3] = tet[i];
+  tet[i] = tet3;
   return i;
 }
 
 /* exception handler for tet_traverse, returns either i0 or i1,
  * depending on which replacement will leave the larger area base triangle
- * is it really necessary? could it just return i0? */
-static int bigger_tri(real xy[][3], int tet[], int i0, int i1)
+ */
+static int
+bigger_tri(real xy[][3], int tet[], int i0, int i1)
 {
-  int j= i0? i0-1 : 2;
-  real x3= xy[tet[3]][0];
-  real y3= xy[tet[3]][1];
+  int j = i0? i0-1 : 2;
+  real x3 = xy[tet[3]][0];
+  real y3 = xy[tet[3]][1];
   real area0, area1;             /* neither will be negative here */
-  int three= (i1==i0);           /* i0=i1=0 means check all three */
-  if (three) i1= 1;
+  int three = (i1==i0);          /* i0=i1=0 means check all three */
+  if (three) i1 = 1;
 
-  area0= (xy[tet[3^i0^j]][0]-x3)*(xy[tet[j]][1]-y3) -
+  area0 = (xy[tet[3^i0^j]][0]-x3)*(xy[tet[j]][1]-y3) -
     (xy[tet[3^i0^j]][1]-y3)*(xy[tet[j]][0]-x3);
-  j= i1? i1-1 : 2;
-  area1= (xy[tet[3^i1^j]][0]-x3)*(xy[tet[j]][1]-y3) -
+  j = i1? i1-1 : 2;
+  area1 = (xy[tet[3^i1^j]][0]-x3)*(xy[tet[j]][1]-y3) -
     (xy[tet[3^i1^j]][1]-y3)*(xy[tet[j]][0]-x3);
-  i0= (area0>area1)? i0 : i1;
-  if (!three) return i0;         /* larger of two */
+  j = (area0>area1)? i0 : i1;
+  if (!three) {
+    /* never knowingly step into a zero area triangle */
+    if (area0<=0. && area1<=0.) return 3-i0-i1;
+    return j;              /* otherwise larger of two */
+  }
 
-  if (i0) area0= area1;
-  area1= (xy[tet[0]][0]-x3)*(xy[tet[1]][1]-y3) -
+  if (j) area0 = area1;
+  area1 = (xy[tet[0]][0]-x3)*(xy[tet[1]][1]-y3) -
     (xy[tet[0]][1]-y3)*(xy[tet[1]][0]-x3);
-  return (area0>area1)? i0 : 2;  /* largest of all three */
+  return (area0>area1)? j : 2;  /* largest of all three */
 }
 
 /* ------------------------------------------------------------------------ */
@@ -96,6 +131,7 @@ real tri_intersect(real xy[][3], int tri[])
   real x2= xy[tri[2]][0];
   real y2= xy[tri[2]][1];
   real z2= xy[tri[2]][2];
+  /* see note in tet_traverse about area==0, roundoff makes it possible */
   real area= (x0-x2)*(y1-y2) - (y0-y2)*(x1-x2);
   x1= x1*y2 - y1*x2;
   x0= x2*y0 - y2*x0;
@@ -122,13 +158,24 @@ int tri_traverse(real qp[], real xy[][3], int tri[], real dot[])
 
 /* ------------------------------------------------------------------------ */
 
-int tri_contains(real xy[][3], int tri[])
+real
+tri_find(real xy[][3], int tri[], real qr2)
 {
-  real cross01= xy[tri[0]][0]*xy[tri[1]][1] - xy[tri[0]][1]*xy[tri[1]][0];
-  real cross12= xy[tri[1]][0]*xy[tri[2]][1] - xy[tri[1]][1]*xy[tri[2]][0];
-  real cross20= xy[tri[2]][0]*xy[tri[0]][1] - xy[tri[2]][1]*xy[tri[0]][0];
-  if (cross01+cross12+cross20 == 0.) return 0;  /* assume area non-negative */
-  return cross01>=0. && cross12>=0. && cross20>=0.;
+  real x0 = xy[tri[0]][0];
+  real y0 = xy[tri[0]][1];
+  real z0 = xy[tri[0]][2];
+  real x1 = xy[tri[1]][0];
+  real y1 = xy[tri[1]][1];
+  real z1 = xy[tri[1]][2];
+  real x2 = xy[tri[2]][0];
+  real y2 = xy[tri[2]][1];
+  real z2 = xy[tri[2]][2];
+  real area = (x0-x2)*(y1-y2) - (y0-y2)*(x1-x2);
+  x1 = x1*y2 - y1*x2;
+  x0 = x2*y0 - y2*x0;
+  /* algorithm identical to tri_intersect to this point */
+  if (x1<0. || x0<0. || area<x0+x1 || !area) return 1.e35;
+  return qr2 * (z2 + ((z0-z2)*x1 + (z1-z2)*x0)/area);
 }
 
 void tri_check(real xy[][3], int tri[])
@@ -273,7 +320,7 @@ int entry_setup(TK_ray *ray, real xy[][3], int tri[],
   flags[2]= (ABS(xk-xj)>dot[3] && (xk-xj<0.)==sign);
   if (flags[2]) {
     /* on entry side always head toward the ray */
-    if ((xk-xj<0.) == xj<0.) sign|= 2;
+    if ((xk-xj<0.) == (xj<0.)) sign|= 2;
   } else {
     /* on exit side head opposite to ray direction unless
      * flag is set for interior boundary -- this is purely a
@@ -464,6 +511,93 @@ int ray_reflect(TK_ray *ray, real xy[][3], int tri[],
   return (qr[2]<0.)==qr2;
 }
 
+int
+ray_certify(TK_ray *ray, real xy[][3], int tri[], int nxy)
+{
+  /* reflection operation must preserve critical topology
+   * specifically, ray must remain inside tri after reflection
+   * - if ray is very near an edge or vertex, roundoff error
+   *   may move reflected ray outside tri, which is very bad
+   */
+  real xa = xy[tri[0]][0];
+  real ya = xy[tri[0]][1];
+  real xb = xy[tri[1]][0];
+  real yb = xy[tri[1]][1];
+  real xc = xy[tri[2]][0];
+  real yc = xy[tri[2]][1];
+  real axb = xa*yb - ya*xb;
+  real bxc = xb*yc - yb*xc;
+  real cxa = xc*ya - yc*xa;
+  if (axb+bxc+cxa <= 0.)
+    return -1;
+
+  if (axb>=0. && bxc>=0. && cxa>=0.) {
+    return 0;
+  } else {
+    /* strategy is to try to step back to nearest edge or vertex */
+    int i;
+    real dx, dy, r, ddx, ddy;
+    volatile real x0, y0, x1, y1, x2, y2;
+    if (axb < 0.) {
+      if (bxc < 0.) {
+        dx = xb; dy = yb;
+      } else if (cxa < 0.) {
+        dx = xa; dy = ya;
+      } else {
+        dy = xa-xb; dx = yb-ya; r = axb/(dx*dx+dy*dy);
+        dx *= r; dy *= r;
+        for (;;) {
+          x0 = xa-dx;  y0 = ya-dy;  x1 = xb-dx;  y1 = yb-dy;
+          if (x0!=xa || y0!=ya || x1!=xb || y1!=yb) break;
+          dx += dx;  dy += dy;
+        }
+      }
+    } else if (bxc < 0.) {
+      if (cxa < 0.) {
+        dx = xc; dy = yc;
+      } else {
+        dy = xb-xc; dx = yc-yb; r = bxc/(dx*dx+dy*dy);
+        dx *= r; dy *= r;
+        for (;;) {
+          x0 = xb-dx;  y0 = yb-dy;  x1 = xc-dx;  y1 = yc-dy;
+          if (x0!=xb || y0!=yb || x1!=xc || y1!=yc) break;
+          dx += dx;  dy += dy;
+        }
+      }
+    } else {
+      dy = xc-xa; dx = ya-yc; r = cxa/(dx*dx+dy*dy);
+      dx *= r; dy *= r;
+      for (;;) {
+        x0 = xc-dx;  y0 = yc-dy;  x1 = xa-dx;  y1 = ya-dy;
+        if (x0!=xc || y0!=yc || x1!=xa || y1!=ya) break;
+        dx += dx;  dy += dy;
+      }
+    }
+
+    ddx = dx;  ddy = dy;
+    for (i=0 ; i<10 ; i++) {
+      x0 = xa - dx;  y0 = ya - dy;
+      x1 = xb - dx;  y1 = yb - dy;
+      x2 = xc - dx;  y2 = yc - dy;
+      axb = x0*y1 - y0*x1;
+      bxc = x1*y2 - y1*x2;
+      cxa = x2*y0 - y2*x0;
+      if (axb+bxc+cxa <=0.)
+        return -1;
+      if (axb>=0. && bxc>=0. && cxa>=0.) break;
+      dx += ddx;  dy+= ddy;
+    }
+    if (i >= 10)
+      return -1;  /* maybe should flag this -2? */
+
+    /* move ray back into tri */
+    ray->p[0] += dx;
+    ray->p[1] += dy;
+    for (i=0 ; i<nxy ; i++) xy[i][0] -= dx, xy[i][1] -= dy;
+    return 1;
+  }
+}
+
 /* ------------------------------------------------------------------------ */
 
 void ray_init(TK_ray *ray, real pa[], real qa[], real matrix[][3])
