@@ -1,46 +1,22 @@
 /*
-   DGECON.C
-   LAPACK routines to estimate the condition number of a matrix.
-
-   $Id$
+ * $Id: dgecon.c,v 1.1.1.1 2005/09/18 22:04:39 dhmunro Exp $
+ * LAPACK routines to estimate the condition number of a matrix.
+ */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
  */
 
 #include "dg.h"
 
-
-/*-----prototypes of functions defined here-----*/
-extern void dlacon( long n, double v[], double x[], long isgn[],
-                   double *est, long *kase );
-extern void dlamc5( long beta, long p, long emin, int ieee,
-                   long *emax, double *rmax );
-extern void dlamc4( long *emin, double start, long base );
-extern double   dlamc3( double a, double b );
-extern void dlamc2( long *beta, long *t, int *rnd, double *eps,
-                   long *emin, double *rmin, long *emax, double *rmax );
-extern void dlamc1( long *beta, long *t, int *rnd, int *ieee1 );
-extern double   dlamch( char cmach );
-extern void dlatrs( char uplo, char trans, char diag, char normin, long n,
-                   double a[], long lda, double x[], double *scale,
-                   double cnorm[], long *info );
-extern void dlabad( double *small, double *large );
-extern void drscl( long n, double sa, double sx[], long incx );
-extern void dgecon( char norm, long n, double a[], long lda, double anorm,
-                   double *rcond, double work[], long iwork[],long *info );
-/*-----end of prototypes-----*/
-
-
-
 /*---blas routines---*/
 /* ddot dcopy dasum idamax daxpy dscal dtrsv */
 
 
-
 /*---converted nutty string switches to single characters (lower case)---*/
 #define lsame(x,y) ((x)==(y))
 
-extern void xerbla(char *,long);
-
-
 
 /*-----Fortran intrinsics converted-----*/
 extern double pow(double,double);  /* performance never an issue here! */
@@ -57,7 +33,6 @@ extern double log10(double);
 /*-----end of Fortran intrinsics-----*/
 
 
-
 void dgecon( char norm, long n, double a[], long lda, double anorm,
             double *rcond, double work[], long iwork[],long *info )
 {
@@ -2154,9 +2129,9 @@ void dlacon( long n, double v[], double x[], long isgn[],
    result to pass through a memory location.
    Actually, it would be more correct to go after the comparison
    operation than the sum, but this perturbs the existing code less.  */
-extern void   dlamc3_worker( double a, double b );
+extern void dlamc3_worker( double a, double b );
 extern double dlamc3_sum;
-void   dlamc3_worker( double a, double b )
+void dlamc3_worker( double a, double b )
 {
   dlamc3_sum= a+b;
 }
