@@ -1,71 +1,20 @@
 /*
-   DGELSS.C
-   LAPACK matrix solver using SVD.
-
-   $Id$
+ * $Id: dgelss.c,v 1.1.1.1 2005/09/18 22:04:43 dhmunro Exp $
+ * LAPACK matrix solver using SVD.
+ */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
  */
 
 #include "dg.h"
 
 
-/*-----prototypes of functions defined here-----*/
-extern void dgelss( long m, long n, long nrhs, double a[], long lda,
-                   double b[], long ldb, double s[], double rcond,
-                   long *rank,double work[], long lwork, long *info );
-/*-----end of prototypes-----*/
-
-
-
-
-/*---other lapack routines---*/
-extern double   dlange( char norm, long m, long n, double a[], long lda,
-                       double work[] );
-extern double   dlamch( char cmach );
-extern void dlabad( double *small, double *large );
-extern long ilaenv( long ispec, char *name, char *opts,
-                   long n1, long n2, long n3,long n4 );
-
-extern void drscl( long n, double sa, double sx[], long incx );
-extern void dormqr( char side, char trans, long m, long n, long k,
-                   double a[], long lda, double tau[], double c[], long ldc,
-                   double work[], long lwork, long *info );
-extern void dormlq( char side, char trans, long m, long n, long k,
-                   double a[], long lda, double tau[], double c[], long ldc,
-                   double work[], long lwork, long *info );
-extern void dgelqf( long m, long n, double a[], long lda, double tau[],
-                   double work[], long lwork, long *info );
-extern void dgeqrf( long m, long n, double a[], long lda, double tau[],
-                   double work[], long lwork, long *info );
-extern void dlaset( char uplo, long m, long n, double alpha, double beta,
-                   double a[], long lda );
-extern void dlacpy( char uplo, long m, long n, double a[], long lda,
-                   double b[], long ldb );
-extern void dlascl( char type, long kl, long ku, double cfrom, double cto,
-                   long m, long n, double a[], long lda, long *info );
-extern void dgebrd( long m, long n, double a[], long lda,
-                   double d[], double e[], double tauq[], double taup[],
-                   double work[], long lwork,long *info );
-extern void dormbr( char vect, char side, char trans, long m, long n, long k,
-                   double a[], long lda, double tau[], double c[],long ldc,
-                   double work[], long lwork, long *info );
-extern void dorgbr( char vect, long m, long n, long k, double a[], long lda,
-                   double tau[], double work[], long lwork, long *info );
-extern void dbdsqr( char uplo, long n, long ncvt, long nru, long ncc,
-                   double d[], double e[], double vt[], long ldvt,
-                   double u[],long ldu, double c[], long ldc,
-                   double work[], long *info );
-
-
-
 /*---blas routines---*/
 /* dcopy dgemv dgemm */
 
 
-
-extern void xerbla(char *,long);
-
-
-
 /*-----Fortran intrinsics converted-----*/
 #define min(x,y) ((x)<(y)?(x):(y))
 #define max(x,y) ((x)>(y)?(x):(y))
