@@ -1,86 +1,22 @@
 /*
-   DLASR.C
-   LAPACK matrix solver using SVD (split from dgelss.c).
-
-   $Id$
+ * $Id: dlasr.c,v 1.1.1.1 2005/09/18 22:04:50 dhmunro Exp $
+ * LAPACK matrix solver using SVD (split from dgelss.c).
+ */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
  */
 
 #include "dg.h"
 
 
-/*-----prototypes of functions defined here-----*/
-extern void dlacpy( char uplo, long m, long n, double a[], long lda,
-                   double b[], long ldb );
-extern void dlabrd( long m, long n, long nb, double a[], long lda,
-                   double d[], double e[], double tauq[], double taup[],
-                   double x[], long ldx, double y[],long ldy );
-extern void dgebd2( long m, long n, double a[], long lda,
-                   double d[], double e[], double tauq[], double taup[],
-                   double work[], long *info );
-extern void dgebrd( long m, long n, double a[], long lda,
-                   double d[], double e[], double tauq[], double taup[],
-                   double work[], long lwork,long *info );
-extern void dormbr( char vect, char side, char trans, long m, long n, long k,
-                   double a[], long lda, double tau[], double c[],long ldc,
-                   double work[], long lwork, long *info );
-extern void dorg2r( long m, long n, long k, double a[], long lda,
-                   double tau[], double work[], long *info );
-extern void dorgqr( long m, long n, long k, double a[], long lda,
-                   double tau[], double work[], long lwork, long *info );
-extern void dorgl2( long m, long n, long k, double a[], long lda,
-                   double tau[], double work[], long *info );
-extern void dorglq( long m, long n, long k, double a[], long lda,
-                   double tau[], double work[], long lwork, long *info );
-extern void dorgbr( char vect, long m, long n, long k, double a[], long lda,
-                   double tau[], double work[], long lwork, long *info );
-extern void dlartg( double f, double g, double *cs, double *sn, double *r );
-extern void dlasv2( double f, double g, double h,
-                   double *ssmin, double *ssmax, double *snr,
-                   double *csr, double *snl, double *csl );
-extern void dlas2( double f, double g, double h,
-                  double *ssmin, double *ssmax );
-extern void dlasr( char side, char pivot, char direct, long m, long n,
-                  double c[], double s[], double a[], long lda );
-/*-----end of prototypes-----*/
-
-
-
-
-/*---other lapack routines---*/
-extern long ilaenv( long ispec, char *name, char *opts,
-                   long n1, long n2, long n3,long n4 );
-
-extern void dlarft( char direct, char storev, long n, long k,
-                   double v[], long ldv, double tau[], double t[], long ldt );
-extern void dlarfb( char side, char trans, char direct, char storev,
-                   long m, long n, long k, double v[], long ldv,
-                   double t[], long ldt, double c[], long ldc,
-                   double work[], long ldwork );
-extern void dlarfg( long n, double *alpha, double x[], long incx,
-                   double *tau );
-extern void dlarf( char side, long m, long n, double v[], long incv,
-                  double tau, double c[], long ldc, double work[] );
-extern void dormqr( char side, char trans, long m, long n, long k,
-                   double a[], long lda, double tau[], double c[], long ldc,
-                   double work[], long lwork, long *info );
-extern void dormlq( char side, char trans, long m, long n, long k,
-                   double a[], long lda, double tau[], double c[], long ldc,
-                   double work[], long lwork, long *info );
-
-
-
 /*---blas routines---*/
 /* dscal dgemv dgemm */
 
-
-
 /*---converted nutty string switches to single characters (lower case)---*/
 #define lsame(x,y) ((x)==(y))
 
-extern void xerbla(char *,long);
-
-
-
 /*-----Fortran intrinsics converted-----*/
 #define abs(x) ((x)>=0?(x):-(x))
 extern double sqrt(double);
