@@ -1,8 +1,11 @@
 /*
- * handler.c -- $Id$
+ * $Id: handler.c,v 1.1.1.1 2005/09/18 22:05:39 dhmunro Exp $
  * exception handling for UNIX machines
- *
- * Copyright (c) 1998.  See accompanying LEGAL file for details.
+ */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
  */
 
 #ifndef _POSIX_SOURCE
@@ -88,3 +91,8 @@ u_sigalrm(int sig)
   signal(sig, &u_sigalrm);
   if (p_signalling==PSIG_INT) p_abort();
 }
+
+#ifdef NO_XLIB
+/* FIXME this is a hack - not quite correct because of prepolling... */
+void p_qclear(void) {}
+#endif
