@@ -1,8 +1,11 @@
 /*
- * usernm.c -- $Id$
+ * $Id: usernm.c,v 1.2 2005/11/12 04:21:56 dhmunro Exp $
  * p_getuser for UNIX machines
- *
- * Copyright (c) 1998.  See accompanying LEGAL file for details.
+ */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
  */
 
 #ifndef _POSIX_SOURCE
@@ -30,7 +33,7 @@ p_getuser(void)
 }
 
 #else
-
+# ifndef NO_CUSERID
 extern char *cuserid(char *);
 char *
 p_getuser(void)
@@ -38,5 +41,13 @@ p_getuser(void)
   char *user = cuserid((char *)0);
   return user;
 }
-
+# else
+extern char *getenv(char *);
+char *
+p_getuser(void)
+{
+  char *user = getenv("LOGNAME");
+  return user;
+}
+# endif
 #endif
