@@ -1,8 +1,13 @@
 /*
-    ARRAY.C
-
-    $Id: array.c,v 1.1 1993/08/27 18:32:09 munro Exp $
+ * $Id: array.c,v 1.2 2005/11/13 21:01:56 dhmunro Exp $
+ */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
+ */
 
+/*
     Implement Yorick array indexing.  This is one half of the Yorick
     Eval instruction -- the other half of Eval is function invocation,
     which is handled in fnctn.c.  An expression of the form:
@@ -47,8 +52,6 @@
     same origin as the input array, and an index list produces result
     dimensions with origins copied from the index list dimensions.
  */
-/*    Copyright (c) 1994.  The Regents of the University of California.
-                    All rights reserved.  */
 
 #include "ydata.h"
 
@@ -57,7 +60,7 @@ extern UnaryOp EvalAY, EvalLV;
 
 extern Operand *FormOperandDB(Symbol *owner, Operand *op);
 
-#define MAX_INDICES 10
+#define MAX_INDICES (Y_DIMSIZE-1)
 
 /*
    For several types of DataBlock, the Eval operation means to index
@@ -426,7 +429,7 @@ static void DotdotScan(Symbol *stack, int singleIndex)
   number= 1;
   while (iDummy < nDummy-na) {
     n= dims[iDummy]->number;
-    if (overOrigin) overOrigin/= n;
+    if (overOrigin) overOrigin= (overOrigin+n-1)/n;
     number*= n;
     /* copy origin here */
     if (!singleIndex)
