@@ -1,11 +1,12 @@
 /*
-    BINIO.C
-    Define Yorick functions for dealing with binary I/O
-
-    $Id: binio.c,v 1.1 1993/08/27 18:32:09 munro Exp $
+ * $Id: binio.c,v 1.4 2010/02/28 21:52:28 dhmunro Exp $
+ * Define Yorick functions for dealing with binary I/O
+ */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
  */
-/*    Copyright (c) 1994.  The Regents of the University of California.
-                    All rights reserved.  */
 
 #ifdef NOT_YORICK
 #include "binio.h"
@@ -168,7 +169,7 @@ StructDef *AddStruct(IOStream *file, const char *name, long n)
     if (file->history) {
       HistoryInfo *history= file->history;
       if (history->nRecords>1)
-	YError("illegal to add struct to file after 1st record");
+        YError("illegal to add struct to file after 1st record");
       if (file!=history->child) file= history->child;
     }
 
@@ -176,9 +177,9 @@ StructDef *AddStruct(IOStream *file, const char *name, long n)
       StructDef **structList= file->structList;
       base= structList[hashIndex];
       if (base->references || (base->model!=&stringStruct &&
-			       base->model!=&pointerStruct)) return 0;
+                               base->model!=&pointerStruct)) return 0;
       /* redefinition of unreferenced default string or pointer
-	 is the exception -- it is OK */
+         is the exception -- it is OK */
       Unref(base->model);
       base->model= 0;
       base->dataOps= 0;
@@ -196,7 +197,7 @@ StructDef *AddStruct(IOStream *file, const char *name, long n)
     if (HashAdd(&yStructTable, name, 0L)) {
       StructDef *old= yStructList[hashIndex];
       if (hashIndex<=8)
-	YError("attempt to replace in-memory StructDef of basic data type");
+        YError("attempt to replace in-memory StructDef of basic data type");
       yStructList[hashIndex]= 0;
       Unref(old);
     } else {
@@ -228,8 +229,8 @@ static char *primNames[N_PRIMITIVES]= {
  */
 
 int DefinePrimitive(StructDef *base, long size, int alignment,
-		    int addressType, int order, FPLayout *fpLayout,
-		    StructDef *model, Converter *Convert)
+                    int addressType, int order, FPLayout *fpLayout,
+                    StructDef *model, Converter *Convert)
 {
   int i;
 
@@ -243,7 +244,7 @@ int DefinePrimitive(StructDef *base, long size, int alignment,
     if (base->index<NDX_STRING && base->index!=i) return 1;
     /* redefinition of char, short, int, long, float, or double */
     else if ((i==NDX_STRING || i==NDX_POINTER) && base->index!=i &&
-	     !Convert) return 2;
+             !Convert) return 2;
     /* illegal to alias another type to string or pointer */
   } else {
     i= N_PRIMITIVES;
@@ -264,11 +265,11 @@ int DefinePrimitive(StructDef *base, long size, int alignment,
     else if (i==NDX_STRING)              base->Convert= &ConvertQ;
     else if (i<NDX_STRING && i>NDX_LONG)
       base->Convert= (base->order==model->order && base->size==model->size &&
-		      SameFPLayout(base->fpLayout, model->fpLayout))?
-	0 : &ConvertF;
+                      SameFPLayout(base->fpLayout, model->fpLayout))?
+        0 : &ConvertF;
     else if (i<=NDX_LONG && i>0)
       base->Convert=
-	(base->order==model->order && base->size==model->size)? 0 : &ConvertI;
+        (base->order==model->order && base->size==model->size)? 0 : &ConvertI;
     else if (i==0)                       base->Convert= 0;
     else if (model->size!=size) return 3;
     /* model size differs--require Converter */
@@ -292,22 +293,22 @@ int DefinePrimitive(StructDef *base, long size, int alignment,
       IOStream *file= base->file;
       long nStructs= file->structTable.nItems;
       if (nStructs>NDX_STRING) {
-	model= file->structList[NDX_STRING];
-	if (model->Convert==&ConvertQ) {
-	  model->size= base->size;
-	  model->alignment= base->alignment;
-	  model->addressType= base->addressType;
-	  model->order= base->order;
-	}
+        model= file->structList[NDX_STRING];
+        if (model->Convert==&ConvertQ) {
+          model->size= base->size;
+          model->alignment= base->alignment;
+          model->addressType= base->addressType;
+          model->order= base->order;
+        }
       }
       if (nStructs>NDX_POINTER) {
-	model= file->structList[NDX_POINTER];
-	if (model->Convert==&ConvertP) {
-	  model->size= base->size;
-	  model->alignment= base->alignment;
-	  model->addressType= base->addressType;
-	  model->order= base->order;
-	}
+        model= file->structList[NDX_POINTER];
+        if (model->Convert==&ConvertP) {
+          model->size= base->size;
+          model->alignment= base->alignment;
+          model->addressType= base->addressType;
+          model->order= base->order;
+        }
       }
     }
 
@@ -340,7 +341,7 @@ int DefinePrimitive(StructDef *base, long size, int alignment,
 }
 
 int AddMember(StructDef *base, long offset, const char *name,
-	      StructDef *memType, Dimension *dims)
+              StructDef *memType, Dimension *dims)
 {
   long number, index;
   IOStream *file= base->file;
@@ -362,8 +363,8 @@ int AddMember(StructDef *base, long offset, const char *name,
 
   /* fill in new member  */
   memType= CopyStruct(file, memType);  /* allows memType to be a
-					  memory struct as well as
-					  a struct for this file */
+                                          memory struct as well as
+                                          a struct for this file */
   if (!memType) return 2;
   base->members[index].base= Ref(memType);
   base->members[index].dims= Ref(dims);
@@ -371,7 +372,7 @@ int AddMember(StructDef *base, long offset, const char *name,
 
   /* NOTE-- base->size is NOT necessarily correct while the struct
             is being built.  InstallStruct adjusts base->size to be
-	    a multiple of base->alignment if necessary.  */
+            a multiple of base->alignment if necessary.  */
   if (offset<0) {
     /* must compute offset/alignment for the new data, which goes at
        next available offset in base */
@@ -452,30 +453,30 @@ void InstallStruct(StructDef *base, StructDef *model)
        to this StructDef.  Check yStructTable for this name to see if
        any in-memory structs have the right name.  */
     if (!model && !base->model &&
-	HashFind(&yStructTable, StructName(base), 0L)) {
+        HashFind(&yStructTable, StructName(base), 0L)) {
       model= yStructList[hashIndex];
       if (!EquivStruct(base, model)) model= 0;
     }
 
     if (base->model) {
       /* Caller has already filled in base->model -- presume everything
-	 else (Convert, order, fpLayout) is also correct.  */
+         else (Convert, order, fpLayout) is also correct.  */
       model= base->model;
 
     } else if (model) {
       /* An equivalent in-memory model for this StructDef already exists
-	 and has the same name -- use it.  */
+         and has the same name -- use it.  */
       if (!base->table.nItems) {
-	DefinePrimitive(base, base->size, base->alignment,
-			base->addressType, base->order, base->fpLayout,
-			model, (Converter *)0);
+        DefinePrimitive(base, base->size, base->alignment,
+                        base->addressType, base->order, base->fpLayout,
+                        model, (Converter *)0);
 
       } else {
-	base->model= Ref(model);
-	/* AddMember will not catch on to the fact that a Converter is
-	   necessary if the primitive alignments differ.  */
-	if (!base->Convert && !StructEqual(base, model))
-	  base->Convert= &ConvertS;
+        base->model= Ref(model);
+        /* AddMember will not catch on to the fact that a Converter is
+           necessary if the primitive alignments differ.  */
+        if (!base->Convert && !StructEqual(base, model))
+          base->Convert= &ConvertS;
       }
 
     } else if (base->table.nItems || (base->order==0 && base->size!=1)) {
@@ -486,37 +487,37 @@ void InstallStruct(StructDef *base, StructDef *model)
       StructDef *submodel;
 
       /* Don't want to call AddStruct here, as this would replace an
-	 existing in-memory data structure of the same name.  However,
-	 if no such in-memory struct exists, go ahead and make one now.  */
+         existing in-memory data structure of the same name.  However,
+         if no such in-memory struct exists, go ahead and make one now.  */
       if (HashAdd(&yStructTable, StructName(base), 0L)) {
-	base->model= model= NewStructDef((IOStream *)0, hashIndex);
+        base->model= model= NewStructDef((IOStream *)0, hashIndex);
       } else {
-	HASH_MANAGE(yStructTable, StructDef *, yStructList);
-	base->model= model= NewStructDef((IOStream *)0, hashIndex);
-	yStructList[hashIndex]= Ref(model);
+        HASH_MANAGE(yStructTable, StructDef *, yStructList);
+        base->model= model= NewStructDef((IOStream *)0, hashIndex);
+        yStructList[hashIndex]= Ref(model);
       }
 
       for (i=0 ; i<nItems ; i++) {
-	submodel= members[i].base->model;
-	while (submodel->model) submodel= submodel->model;
-	if (AddMember(model, -1L, names[i], submodel, members[i].dims))
-	  YError("bad StructDef passed to InstallStruct");
+        submodel= members[i].base->model;
+        while (submodel->model) submodel= submodel->model;
+        if (AddMember(model, -1L, names[i], submodel, members[i].dims))
+          YError("bad StructDef passed to InstallStruct");
       }
       if (!nItems) {
-	model->size= base->size;
-	model->alignment= base->alignment;
+        model->size= base->size;
+        model->alignment= base->alignment;
       }
       InstallStruct(model, 0);
 
       /* AddMember will not catch on to the fact that a Converter is
-	 necessary if the primitive alignments differ.  */
+         necessary if the primitive alignments differ.  */
       if (!base->Convert && !StructEqual(base, model))
-	base->Convert= &ConvertS;
+        base->Convert= &ConvertS;
 
     } else {
       DefinePrimitive(base, base->size, base->alignment,
-		      base->addressType, base->order, base->fpLayout,
-		      (StructDef *)0, (Converter *)0);
+                      base->addressType, base->order, base->fpLayout,
+                      (StructDef *)0, (Converter *)0);
       model= base->model;
     }
 
@@ -555,15 +556,15 @@ int EquivStruct(StructDef *fil, StructDef *mem)
       Member *mm= mem->members;
       Dimension *fd, *md;
       for (i=0 ; i<nItems ; i++) {
-	if (!EquivStruct(fm[i].base, mm[i].base)) return 0;
-	fd= fm[i].dims;
-	md= mm[i].dims;
-	while (fd && md) {
-	  if (fd->number != md->number) return 0;
-	  fd= fd->next;
-	  md= md->next;
-	}
-	if (fd || md) return 0;
+        if (!EquivStruct(fm[i].base, mm[i].base)) return 0;
+        fd= fm[i].dims;
+        md= mm[i].dims;
+        while (fd && md) {
+          if (fd->number != md->number) return 0;
+          fd= fd->next;
+          md= md->next;
+        }
+        if (fd || md) return 0;
       }
       return 1;
     }
@@ -571,7 +572,7 @@ int EquivStruct(StructDef *fil, StructDef *mem)
 }
 
 int AddVariable(IOStream *file, long address, const char *name,
-		StructDef *varType, Dimension *dims)
+                StructDef *varType, Dimension *dims)
 {
   long number, index;
   HistoryInfo *history= file->history;
@@ -583,7 +584,7 @@ int AddVariable(IOStream *file, long address, const char *name,
     if (history->nRecords>1)
       YError("illegal to add record variable to file after 1st record");
     else if (history->nRecords>0 && history->recNumber>=0 &&
-	     file->nextAddress-file->offset > history->recordSize)
+             file->nextAddress-file->offset > history->recordSize)
       YError("non-contiguous record: be careful with pointers/strings?");
   }
 
@@ -600,8 +601,8 @@ int AddVariable(IOStream *file, long address, const char *name,
 
   /* fill in new member  */
   varType= CopyStruct(file, varType);  /* allows varType to be a
-					  memory struct as well as
-					  a struct for this file */
+                                          memory struct as well as
+                                          a struct for this file */
   if (!varType) return 2;
   file->types[index].base= Ref(varType);
   file->types[index].dims= Ref(dims);
@@ -611,8 +612,8 @@ int AddVariable(IOStream *file, long address, const char *name,
     /* must compute address/alignment for the new data, which goes at
        next available address in file */
     long next= AlignAdjust(history? history->recordSize : file->nextAddress,
-			   file->dataAlign? file->dataAlign :
-			                    varType->alignment);
+                           file->dataAlign? file->dataAlign :
+                                            varType->alignment);
     file->addresses[index]= next;
     next+= varType->size*number;
     if (history) history->recordSize= next;
@@ -637,10 +638,14 @@ int AddVariable(IOStream *file, long address, const char *name,
 HistoryInfo *AddHistory(IOStream *file, long size)
 {
   IOStream *child;
-  HistoryInfo *history= p_malloc(sizeof(HistoryInfo));
+  HistoryInfo *history;
   long nStructs, i;
   StructDef **structList, *base;
 
+  if (y_vopen_file(file->stream))
+    YError("vopen binary file handles do not support history records");
+
+  history= p_malloc(sizeof(HistoryInfo));
   history->parent= file;
   history->child= 0;  /* temporary */
   history->nFamily= 1;
@@ -693,7 +698,7 @@ HistoryInfo *AddHistory(IOStream *file, long size)
 }
 
 int AddRecord(HistoryInfo *history,
-	      int flags, double time, long ncyc, long address)
+              int flags, double time, long ncyc, long address)
 {
   long nRecords= history->nRecords;
   IOStream *child= history->child;
@@ -702,26 +707,26 @@ int AddRecord(HistoryInfo *history,
   if (nRecords>0) {
     /* all records must agree on whether time and ncyc present */
     if (((history->time!=0)^((flags&1)!=0)) ||
-	((history->ncyc!=0)^((flags&2)!=0))) return 1;
+        ((history->ncyc!=0)^((flags&2)!=0))) return 1;
 
   } else {
     if (alignment<=0) {
       /* struct-like alignment is difficult for records, since most
-	 restrictive alignment information has not been kept --
-	 Indeed, the child->dataTable may not yet have been generated
-	 at the time the first record is declared (now).
-	 The best we can do is to assume the most restrictive alignment
-	 of any StructDef which has already been defined.
-	 A "custom" struct definition with more restrictive alignment
-	 may not follow this call, and a specification of record alignment
-	 which is less restrictive that the following guess must be done
-	 as a special case by setting child->dataAlign before calling
-	 this routine.  */
+         restrictive alignment information has not been kept --
+         Indeed, the child->dataTable may not yet have been generated
+         at the time the first record is declared (now).
+         The best we can do is to assume the most restrictive alignment
+         of any StructDef which has already been defined.
+         A "custom" struct definition with more restrictive alignment
+         may not follow this call, and a specification of record alignment
+         which is less restrictive that the following guess must be done
+         as a special case by setting child->dataAlign before calling
+         this routine.  */
       long i, n= child->structTable.nItems;
       StructDef **structList= child->structList;
       for (i=0 ; i<n ; i++)
-	if (structList[i]->alignment>alignment)
-	  alignment= structList[i]->alignment;
+        if (structList[i]->alignment>alignment)
+          alignment= structList[i]->alignment;
       history->recordAlign= alignment;
     }
 
@@ -751,9 +756,9 @@ int AddRecord(HistoryInfo *history,
     address= AlignAdjust(child->nextAddress, alignment);
     after= address + history->recordSize;
     if (nRecords>0 && after>history->fileSize &&
-	history->ifile[nRecords-1]==history->fileNumber) {
+        history->ifile[nRecords-1]==history->fileNumber) {
       if (AddNextFile(history, (char *)0, 1))
-	YErrorIO("failed to create next member of file family");
+        YErrorIO("failed to create next member of file family");
       address= child->nextAddress;
       after= address + history->recordSize;
     } else if (nRecords>0 && (child->permissions&2)) {
@@ -797,7 +802,7 @@ IOOperations defaultIOops= {
    at a time.  Since IOStream contains several pointers, the alignment
    of an IOStream must be at least as strict as a void*.  */
 static MemryBlock iosBlock= {0, 0, sizeof(IOStream),
-				 16*sizeof(IOStream)};
+                                 16*sizeof(IOStream)};
 
 IOFileLink *yBinaryFiles= 0;
 
@@ -887,14 +892,20 @@ void FreeIOStream(void *ios)
       parent= history->parent;
       if (io->fullname==parent->fullname) io->fullname= 0;
       if (stream==parent->stream) stream= 0;
-      if (io->CloseHook) io->CloseHook(io);
+      if (io->CloseHook) {
+        io->CloseHook(io);
+        io->CloseHook = parent->CloseHook = 0;
+      }
       /* if io->ioOps->Close will be called for parent only, flush child */
-      if (!stream) FlushFile(io, 1);
+      if (!stream && io->stream) FlushFile(io, 1);
     }
     io->history= 0;
 
   } else {
-    if (io->CloseHook) io->CloseHook(io);
+    if (io->CloseHook) {
+      io->CloseHook(io);
+      io->CloseHook = 0;
+    }
   }
 
   /* closing the file discards any cache blocks */
@@ -930,12 +941,12 @@ void FreeIOStream(void *ios)
     structList[i]= 0;
     if (base->references) {
       if (!deadFile.ops) {
-	deadFile.ops= yOpsStream;
-	deadFile.ioOps= &defaultIOops;
-	deadFile.blockSize= 0x3fff;  /* why? */
-	deadFile.structAlign= 1;     /* why? */
+        deadFile.ops= yOpsStream;
+        deadFile.ioOps= &defaultIOops;
+        deadFile.blockSize= 0x3fff;  /* why? */
+        deadFile.structAlign= 1;     /* why? */
       }
-      base->file= Ref(&deadFile);
+      base->file= RefNC(&deadFile);
     }
     Unref(base);
   }
@@ -1041,10 +1052,10 @@ int AddNextFile(HistoryInfo *history, char *filename, int create)
    (4) Loop:
          If the increment character is before the beginning of name,
             or if it is not in the range A-Z, a-z, or 0-9, insert the
-	    character "A" and break out of this loop.
+            character "A" and break out of this loop.
          If it is 9 or z or Z, it becomes 0,
-	 else increment it and break out of this loop.
-	 Back up so the increment character becomes the previous character.
+         else increment it and break out of this loop.
+         Back up so the increment character becomes the previous character.
  */
 static char *headName= 0, *tailName= 0;
 char *NextFileName(const char *name)
@@ -1069,7 +1080,7 @@ char *NextFileName(const char *name)
     if (i>0) {
       c= tailName[--i];
       while (i>=0 && ((c<'0'||c>'9') && (c<'a'||c>'z') && (c<'A'||c>'Z')))
-	c= tailName[--i];
+        c= tailName[--i];
     } else {
       i= len-1;
     }
@@ -1081,7 +1092,7 @@ char *NextFileName(const char *name)
   j= i + lenN-len;
   for (;;) {
     if (i<0 || (((c=tailName[i])<'0'||c>'9') &&
-		(c<'a'||c>'z') && (c<'A'||c>'Z'))) {
+                (c<'a'||c>'z') && (c<'A'||c>'Z'))) {
       /* out of decent characters to increment, insert one */
       for (i=lenN ; i>j ; i--) nextName[i+1]= nextName[i];
       nextName[j+1]= 'A';
@@ -1161,9 +1172,9 @@ StructDef *CopyStruct(IOStream *file, StructDef *base)
   /* ...and fill in the members in the copy */
   for (i=0 ; i<n ; i++) {
     AddMember(copy, verbatim? base->offsets[i] : -1L,
-	      names[i],
-	      CopyStruct(file, members[i].base),
-	      members[i].dims);
+              names[i],
+              CopyStruct(file, members[i].base),
+              members[i].dims);
   }
 
   if (verbatim) {
@@ -1254,9 +1265,9 @@ long YReadIO(IOStream *file, void *buf, long size, long n)
     long nio= p_fread(stream, buf, nbytes);
     if (nio<nbytes) {
       if (p_ferror(stream)) {
-	YErrorIO("fread failed in YReadIO (binary file I/O)");
+        YErrorIO("fread failed in YReadIO (binary file I/O)");
       } else {
-	nio/= size;
+        nio/= size;
       }
     } else {
       nio= n;
