@@ -1,11 +1,12 @@
 /*
-   CLOG.C
-   Define routines to handle Contents Log (CLOG) language
-
-   $Id: clog.c,v 1.1 1993/08/27 18:32:09 munro Exp $
+ * $Id: clog.c,v 1.4 2010/02/28 21:52:29 dhmunro Exp $
+ * Define routines to handle Contents Log (CLOG) language
+ */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
  */
-/*    Copyright (c) 1994.  The Regents of the University of California.
-                    All rights reserved.  */
 
 
 #ifdef NOT_YORICK
@@ -120,6 +121,8 @@ void CLupdate(IOStream *file)
   HistoryInfo *history= file->history;
   CLbuffer *clBuffer= file->contentsLog;
 
+  if (file->fullname && y_vopen_file(file->stream)) return;
+
   if (history) {
     file= history->parent;
     if (!clBuffer) clBuffer= history->child->contentsLog;
@@ -146,8 +149,11 @@ int DumpClogFile(IOStream *file, const char *name)
 {
   HistoryInfo *history= file->history;
   CLbuffer clBuffer;
+  p_file *stream;
 
-  p_file *stream= p_fopen(name, "w+");  /* create the Clog file */
+  if (file->fullname && y_vopen_file(file->stream)) return 0;
+
+  stream= p_fopen(name, "w+");  /* create the Clog file */
   if (!stream) return 1;
 
   CLinit(&clBuffer, stream, 1024L, 0);
@@ -979,7 +985,7 @@ static int CLdefine(CLbuffer *clBuffer, IOStream *file)
   base->order= order;
   if (seqFlag) base->addressType= 2;
   if (seqFlag==2) {
-    base->model= Ref(&pointerStruct);
+    base->model= RefNC(&pointerStruct);
     base->Convert= &PDBconvert;
   }
   if (flag) base->fpLayout= MakeFPLayout(&fpLayout, size);
@@ -1393,14 +1399,17 @@ static void CLputIdent(p_file *stream, char *identifier)
         } else {
           i++;
         }
-        p_fputs(stream, p_wkspc.c);
       }
-      sprintf(p_wkspc.c, "%s\"", identifier); p_fputs(stream, p_wkspc.c);
+      /* FIXME: buffer overrun? */
+      sprintf(p_wkspc.c, "%s\"", identifier);
+      p_fputs(stream, p_wkspc.c);
     }
 
   } else {
     /* no need for quotes */
-    sprintf(p_wkspc.c, "%s", identifier); p_fputs(stream, p_wkspc.c);
+    /* FIXME: buffer overrun? */
+    sprintf(p_wkspc.c, "%s", identifier);
+    p_fputs(stream, p_wkspc.c);
   }
 }
 
@@ -1409,7 +1418,8 @@ static void CLputChunk(p_file *stream, char *identifier, long n)
   if (n>=1024) YError("impossibly long identifier in CLputChunk");
   idScratch[0]= '\0';
   strncat(idScratch, identifier, n);
-  sprintf(p_wkspc.c, "%s", idScratch); p_fputs(stream, p_wkspc.c);
+  sprintf(p_wkspc.c, "%s", idScratch);
+  p_fputs(stream, p_wkspc.c);
 }
 
 /* ------------------------------------------------------------------------ */
