@@ -1,13 +1,14 @@
 /*
-   FORTRN.C
-   Yorick routines callable from FORTRAN.
-   Includes C-callable versions of a simple memory management package
-   which is safe against asynchronous interrupts.
-
-   $Id: fortrn.c,v 1.1 1993/08/27 18:32:09 munro Exp $
+ * $Id: fortrn.c,v 1.1.1.1 2005/09/18 22:04:05 dhmunro Exp $
+ * Yorick routines callable from FORTRAN.
+ * Includes C-callable versions of a simple memory management package
+ * which is safe against asynchronous interrupts.
+ */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
  */
-/*    Copyright (c) 1994.  The Regents of the University of California.
-                    All rights reserved.  */
 
 #include "pstdlib.h"
 #include "yasync.h"
