@@ -1,15 +1,16 @@
 /*
-    OPS3.C
-    Implement assignment operations:
-
-    Assign =   Increment += (and ++)   Decrement -= (and --)
-
-    Also, implement struct definition and various Eval operations.
-
-    $Id: ops3.c,v 1.1 1993/08/27 18:32:09 munro Exp $
+ * $Id: ops3.c,v 1.2 2010/02/28 21:52:29 dhmunro Exp $
+ * Implement assignment operations:
+ *
+ *  Assign =   Increment += (and ++)   Decrement -= (and --)
+ *
+ *  Also, implement struct definition and various Eval operations.
+ */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
  */
-/*    Copyright (c) 1994.  The Regents of the University of California.
-                    All rights reserved.  */
 
 #include "bcast.h"
 #include "pstdlib.h"
@@ -385,7 +386,7 @@ void Address(void)
      EXCEPT for nil, which loses a reference... */
   pointer->value.p[0]= value;
   sp->value.db= (DataBlock *)pointer;
-  if (!value) Unref(&nilDB);  /* ...since sp->value.db was &nilDB */
+  if (!value) UnrefNC(&nilDB);  /* ...since sp->value.db was &nilDB */
 }
 
 void Deref(void)
