@@ -1,5 +1,5 @@
 /*
- * $Id: std0.c,v 1.7 2007/03/28 09:10:37 thiebaut Exp $
+ * $Id: std0.c,v 1.14 2010/04/16 05:23:43 dhmunro Exp $
  * Define various standard Yorick built-in functions declared in std.i
  *
  *  See std.i for documentation on the functions defined here.
@@ -31,14 +31,17 @@ char *yVersion= Y_VERSION;
 #endif
 
 extern BuiltIn Y_yorick_init, Y_set_path, Y_reshape, Y_array, Y_structof,
-  Y_dimsof, Y_orgsof, Y_sizeof, Y_numberof, Y_typeof, Y_nameof, Y_is_array,
-  Y_is_func, Y_is_void, Y_is_range, Y_is_struct, Y_is_stream, Y_am_subroutine,
+  Y_dimsof, Y_orgsof, Y_sizeof, Y_numberof, Y_typeof, Y_nameof, Y_identof,
+  Y_is_array, Y_is_scalar, Y_is_vector, Y_is_matrix, Y_is_func, Y_is_void,
+  Y_is_range, Y_is_struct, Y_is_stream, Y_is_integer, Y_is_real, Y_is_complex, 
+  Y_is_numerical, Y_is_string, Y_is_pointer, Y_am_subroutine, Y_unref, Y_swap,
   Y_sin, Y_cos, Y_tan, Y_asin, Y_acos, Y_atan, Y_sinh, Y_cosh, Y_tanh,
   Y_exp, Y_log, Y_log10, Y_sqrt, Y_ceil, Y_floor, Y_abs, Y_sign, Y_conj,
   Y_min, Y_max, Y_sum, Y_avg, Y_allof, Y_anyof, Y_noneof, Y_nallof, Y_where,
   Y_get_cwd, Y_get_home, Y_cd, Y_get_env, Y_get_argv, Y_use_origins;
 
 extern BuiltIn Y_set_site, Y_timer, Y_get_path, Y_get_pkgnames;
+extern BuiltIn Y_round, Y_lround;
 
 /*--------------------------------------------------------------------------*/
 
@@ -58,8 +61,8 @@ static void YInit(char *home, char *site, y_pkg_t **pkgs);
 
 extern int YGetLaunchDir(char *argv0);  /* sysdep.c */
 
-char *yLaunchDir=0, *ySiteDir=0, *yHomeDir=0;
-static char *defaultPath = 0;
+char *yLaunchDir=0, *ySiteDir=0, *yHomeDir=0, *defaultPath = 0;
+char *y_user_dir=0, *y_gist_dir=0;
 
 extern int y_on_idle(void);
 extern void y_on_stdin(char *input_line);
@@ -73,6 +76,10 @@ extern int yBatchMode;  /* may be set with -batch, see std0.c */
 static void *y_mmfail(unsigned long n);
 static void y_on_keyline(char *msg);
 
+static int get_type(Symbol *s);
+/* Returns the type identifier (typeID) of stack symbol S, taking care of
+   L-values and of following references.  Keyword symbols yield -1. */
+
 int
 y_launch(int argc, char *argv[],
          char *home, char *site, y_pkg_t **pkgs)
@@ -133,29 +140,33 @@ YInit(char *home, char *site, y_pkg_t **pkgs)
   /* set startup include file search path:
         yLaunchDir:ySiteDir/i0:ySiteDir/contrib
 
-     the switch to the normal include file search path is done in
-     stdx.i, which is included just before custom.i
+     the switch to the normal include file search path is done in stdx.i
      -- create the default path here:
         .:~/.yorick:~/yorick:~/Yorick:ySiteDir/i:ySiteDir/contrib  */
-  yLaunchDir = p_strcpy(ym_argc>0? ym_argv[0] : 0);
-  if (yLaunchDir) {
-    int i;
-    for (i=0 ; yLaunchDir[i] ; i++);
-    while (i>0 && yLaunchDir[i]!='/') i--;
-    if (yLaunchDir[i]=='/') yLaunchDir[i+1] = '\0';
+  if (ym_argc<1 || strcmp(ym_argv[0],"--no-paths")) {
+    yLaunchDir = p_strcpy(ym_argc>0? ym_argv[0] : 0);
+    if (yLaunchDir) {
+      int i;
+      for (i=0 ; yLaunchDir[i] ; i++);
+      while (i>0 && yLaunchDir[i]!='/') i--;
+      if (yLaunchDir[i]=='/') yLaunchDir[i+1] = '\0';
+    }
+    ySiteDir = site? p_strcpy(site) : p_strcpy("/");
+    YNameToHead(&ySiteDir);
+    yHomeDir = home? p_strcpy(home) : p_strcpy("/");
+    YNameToHead(&yHomeDir);
+    Y_set_site(-1);
+  } else {
+    /* yLaunchDir, ySiteDir, yHomeDir, defaultPath will be set elsewhere
+     * - also y_user_dir, y_gist_dir
+     */
+    Y_set_site(-2);
   }
-  ySiteDir = site? p_strcpy(site) : p_strcpy("/");
-  YNameToHead(&ySiteDir);
-  yHomeDir = home? p_strcpy(home) : p_strcpy("/");
-  YNameToHead(&yHomeDir);
-  Y_set_site(0);
 
-  /* set up to read user customizations
+  /* set up to read i-start customizations
      If the first two arguments are -batch alternate.i, use alternate.i
      -- useful for scripts which must NOT risk user customization.  */
-  if (ym_argc<2 || strcmp(ym_argv[1],"-batch")) {
-    YpPush("custom.i");
-  } else {
+  if (ym_argc>1 && !strcmp(ym_argv[1],"-batch")) {
     yBatchMode= 1;  /* data loaded to zero in task.c */
     if (ym_argc==2) {
       ym_argc= 1;
@@ -222,36 +233,29 @@ Y_get_pkgnames(int nargs)
 }
 
 static char *y_make_ipath(char *ylaunch, char *ysite, char *yhome);
+static void y_set_globvar(const char *globname, const char *value, int tohead);
 
 void
 Y_set_site(int nArgs)
 {
   /* This function should ONLY be called from paths.i, or by YInit  */
-
   char *path;
-  Array *dirName;
-  DataBlock *oldDB;
-  long index;
 
   if (nArgs>2) YError("expecting zero, one, or two arguments");
 
   /* record Y_LAUNCH in global variable */
-  index = Globalize("Y_LAUNCH", 0L);
-  oldDB = globTab[index].ops==&dataBlockSym? globTab[index].value.db : 0;
-  dirName = NewArray(&stringStruct, (Dimension *)0);
-  globTab[index].value.db = (DataBlock *)dirName;
-  globTab[index].ops = &dataBlockSym;
-  Unref(oldDB);
-  dirName->value.q[0] = p_strcpy(yLaunchDir);
+  y_set_globvar("Y_LAUNCH", yLaunchDir, 0);
 
-  if (nArgs>=1) {
+  if (nArgs >= 1) {
+    if (ySiteDir) p_free(ySiteDir);
     ySiteDir = p_strcpy(YGetString(sp-nArgs+1));
     YNameToHead(&ySiteDir);
     if (nArgs==2) {
+      if (yHomeDir) p_free(yHomeDir);
       yHomeDir = p_strcpy(YGetString(sp));
       YNameToHead(&yHomeDir);
     }
-  } else {
+  } else if (nArgs == -1) {
 #ifndef NO_FALLBACK_TEST
     p_file *test = 0;
     if (ySiteDir && ySiteDir[0] && YIsAbsolute(ySiteDir)) {
@@ -287,63 +291,113 @@ Y_set_site(int nArgs)
   }
 
   /* record Y_VERSION in global variable */
-  index = Globalize("Y_VERSION", 0L);
-  oldDB = globTab[index].ops==&dataBlockSym? globTab[index].value.db : 0;
-  dirName = NewArray(&stringStruct, (Dimension *)0);
-  globTab[index].value.db = (DataBlock *)dirName;
-  globTab[index].ops = &dataBlockSym;
-  Unref(oldDB);
-  dirName->value.q[0] = p_strcpy(yVersion);
+  y_set_globvar("Y_VERSION", yVersion, 0);
 
   /* record Y_HOME in global variable */
-  index = Globalize("Y_HOME", 0L);
-  oldDB = globTab[index].ops==&dataBlockSym? globTab[index].value.db : 0;
-  dirName = NewArray(&stringStruct, (Dimension *)0);
+  y_set_globvar("Y_HOME", yHomeDir, 1);
+
+  /* record Y_SITE in global variable */
+  y_set_globvar("Y_SITE", ySiteDir, 1);
+
+  /* note: by default, this is never called from paths.i at all */
+
+  if (nArgs == -1) {
+    /* set the initial include path for paths.i
+     * -- if nArgs>=0, this call is from paths.i, just set defaultPath
+     */
+    path = y_make_ipath(yLaunchDir, ySiteDir, yHomeDir);
+    YpSetPaths(path);
+    p_free(path);
+  }
+
+  if (nArgs > -2) {
+    defaultPath = y_make_ipath(0, ySiteDir, yHomeDir);
+    if (nArgs >= 0) YpSetPaths(defaultPath);  /* from paths.i */
+  }
+  y_set_globvar("Y_USER", y_user_dir, 1);
+  y_set_globvar("Y_GISTDIR", y_gist_dir, 0);  /* used in paths.i */
+}
+
+static void
+y_set_globvar(const char *globname, const char *value, int tohead)
+{
+  /* record Y_HOME in global variable */
+  long index = Globalize(globname, 0L);
+  DataBlock *oldDB =
+    globTab[index].ops==&dataBlockSym? globTab[index].value.db : 0;
+  Array *dirName = NewArray(&stringStruct, (Dimension *)0);
   globTab[index].value.db = (DataBlock *)dirName;
   globTab[index].ops = &dataBlockSym;
   Unref(oldDB);
-  dirName->value.q[0] = p_strcpy(yHomeDir);
-
-  /* record Y_SITE in global variable */
-  index = Globalize("Y_SITE", 0L);
-  oldDB = globTab[index].ops==&dataBlockSym? globTab[index].value.db : 0;
-  dirName = NewArray(&stringStruct, (Dimension *)0);
-  globTab[index].value.db = (DataBlock *)dirName;
-  globTab[index].ops= &dataBlockSym;
-  Unref(oldDB);
-  dirName->value.q[0] = p_strcpy(ySiteDir);
-  YNameToHead(dirName->value.q);
-
-  /* set the initial include path for paths.i */
-  path = y_make_ipath(yLaunchDir, ySiteDir, yHomeDir);
-  YpSetPaths(path);
-  p_free(path);
-
-  defaultPath = y_make_ipath(0, ySiteDir, yHomeDir);
+  dirName->value.q[0] = p_strcpy(value);
+  if (tohead) YNameToHead(dirName->value.q);
 }
 
 static char *
 y_make_ipath(char *ylaunch, char *ysite, char *yhome)
 {
-  /*~/.yorick:~/yorick:~/Yorick:Y_SITE/i:Y_SITE/contrib:Y_SITE/i0:Y_HOME/lib*/
+  /*~/.yorick:~/yorick:~/Yorick:Y_SITE/i:Y_SITE/contrib:Y_SITE/i0:Y_HOME/lib*/\
   char *path1 = 0, *path2 = 0;
-  char *yuser = "~/.yorick:";
-  p_dir *yudir = p_dopen("~/yorick");
-  if (yudir) yuser = "~/yorick:";
-  else if ((yudir = p_dopen("~/Yorick"))) yuser = "~/Yorick:";
-  if (yudir) p_dclose(yudir);
+  if (!y_user_dir) {
+    char *yuser = "~/.yorick/";      /* preferred user directory */
+    p_dir *yudir = p_dopen("~/.yorick");
+    if (!yudir) {
+      yudir = p_dopen("~/yorick");   /* unhidden user directory */
+      if (yudir) yuser = "~/yorick/";
+      else {
+        yudir = p_dopen("~/Library/Yorick");             /* Mac OS X */
+        if (yudir) yuser = "~/Library/Yorick/";
+        else {
+          yudir = p_dopen("~/Application Data/Yorick");  /* Windows */
+          if (yudir) yuser = "~/Application Data/Yorick/";
+          else {
+            yudir = p_dopen("~/Yorick");  /* obsolete choice */
+            if (yudir) yuser = "~/Yorick/";
+          }
+        }
+      }
+    }
+    if (yudir) p_dclose(yudir);
+    y_user_dir = p_strcpy(yuser);
+  }
+  YNameToHead(&y_user_dir);
+  if (!y_gist_dir) {
+    p_dir *yudir;
+    y_gist_dir = p_strncat(y_user_dir, "gist", 0);
+    yudir = p_dopen(y_gist_dir);
+    if (!yudir) {  /* for backwards compatibility */
+      char *yuser = "~/.gist";
+      yudir = p_dopen(yuser);
+      if (!yudir) {
+        yuser = "~/gist";
+        yudir = p_dopen(yuser);
+        if (!yudir) {
+          yuser = "~/Gist";
+          yudir = p_dopen(yuser);
+        }
+      }
+      if (yudir) {
+        p_dclose(yudir);
+        p_free(y_gist_dir);
+        y_gist_dir = p_strcpy(yuser);
+      }
+    }
+  }
   if (ylaunch && ylaunch[0]) {
     /* prepend Y_LAUNCH for paths.i only */
     long len = strlen(ylaunch);
-    path1 = (ylaunch[len-1]=='/')? p_strncat(0, ylaunch, len-1) :
+    path2 = (ylaunch[len-1]=='/')? p_strncat(0, ylaunch, len-1) :
       p_strcpy(ylaunch);
-    path2 = p_strncat(path1, PATH_SEP, 0);
-    p_free(path1);
+    path1 = p_strncat(path2, PATH_SEP, 0);
+    p_free(path2);
   } else {
-    path2 = p_strcpy("." PATH_SEP);
+    path1 = p_strcpy("." PATH_SEP);
   }
-  path1 = p_strncat(path2, yuser, 0);
-  if (path2) p_free(path2);
+  path2 = p_strncat(path1, y_user_dir, 0);
+  if (path1) p_free(path1);
+  if (path2[strlen(path2)-1] == '/') path2[strlen(path2)-1] = '\0';
+  path1 = p_strncat(path2, PATH_SEP, 0);
+  p_free(path2);
   path2 = p_strncat(path1, ysite, 0);
   p_free(path1);
   path1 = p_strncat(path2, "i" PATH_SEP, 0);
@@ -775,6 +829,83 @@ void Y_is_array(int nArgs)
   PushIntValue(isArray);
 }
 
+void Y_is_scalar(int nargs)
+{
+  Symbol *s;
+  int result;
+
+  if (nargs != 1) YError("is_scalar takes exactly one argument");
+  s = sp;
+  for (;;) {
+    if (s->ops == &dataBlockSym) {
+      Operand op;
+      Operations *ops =  s->value.db->ops;
+      result = ((ops->isArray || ops == &lvalueOps)
+		&& s->ops->FormOperand(s, &op)->type.dims == (Dimension *)0);
+      break;
+    } else if (s->ops != &referenceSym) {
+      /* Must be one of: intScalar, longScalar, or doubleScalar. */
+      result = 1;
+      break;
+    }
+    s = &globTab[s->index];
+  }
+  PushIntValue(result);
+}
+
+void Y_is_vector(int nargs)
+{
+  Symbol *s;
+  int result;
+
+  if (nargs != 1) YError("is_vector takes exactly one argument");
+  s = sp;
+  for (;;) {
+    if (s->ops == &dataBlockSym) {
+      Operand op;
+      Dimension *dims;
+      Operations *ops =  s->value.db->ops;
+      result = ((ops->isArray || ops == &lvalueOps)
+		&& (dims = s->ops->FormOperand(s, &op)->type.dims) != (Dimension *)0
+		&& dims->next == (Dimension *)0);
+      break;
+    } else if (s->ops != &referenceSym) {
+      /* Must be one of: intScalar, longScalar, or doubleScalar. */
+      result = 0;
+      break;
+    }
+    s = &globTab[s->index];
+  }
+  PushIntValue(result);
+}
+
+void Y_is_matrix(int nargs)
+{
+  Symbol *s;
+  int result;
+
+  if (nargs != 1) YError("is_matrix takes exactly one argument");
+  s = sp;
+  for (;;) {
+    if (s->ops == &dataBlockSym) {
+      Operand op;
+      Dimension *dims;
+      Operations *ops =  s->value.db->ops;
+      result = ((ops->isArray || ops == &lvalueOps)
+		&& (dims = s->ops->FormOperand(s, &op)->type.dims) != (Dimension *)0
+		&& dims->next != (Dimension *)0
+                && dims->next->next == (Dimension *)0);
+      break;
+    } else if (s->ops != &referenceSym) {
+      /* Must be one of: intScalar, longScalar, or doubleScalar. */
+      result = 0;
+      break;
+    }
+    s = &globTab[s->index];
+  }
+  PushIntValue(result);
+}
+
 void Y_is_func(int nArgs)
 {
   Symbol *s= sp;
@@ -883,6 +1014,145 @@ void Y_is_stream(int nArgs)
   PushIntValue(isStream);
 }
 
+void Y_is_integer(int nargs)
+{
+  int type;
+  if (nargs != 1) YError("is_integer takes exactly one argument");
+  type = get_type(sp);
+  PushIntValue(type >= T_CHAR && type <= T_LONG);
+}
+
+void Y_is_real(int nargs)
+{
+  int type;
+  if (nargs != 1) YError("is_real takes exactly one argument");
+  type = get_type(sp);
+  PushIntValue(type == T_DOUBLE || type == T_FLOAT);
+}
+
+void Y_is_complex(int nargs)
+{
+  if (nargs != 1) YError("is_complex takes exactly one argument");
+  PushIntValue(get_type(sp) == T_COMPLEX);
+}
+
+void Y_is_numerical(int nargs)
+{
+  int result;
+  if (nargs != 1) YError("is_numerical takes exactly one argument");
+  switch (get_type(sp)) {
+  case T_CHAR:
+  case T_SHORT:
+  case T_INT:
+  case T_LONG:
+  case T_FLOAT:
+  case T_DOUBLE:
+  case T_COMPLEX:
+    result = 1;
+    break;
+  default:
+    result = 0;
+  }
+  PushIntValue(result);
+}
+
+void Y_is_string(int nargs)
+{
+  if (nargs != 1) YError("is_string takes exactly one argument");
+  PushIntValue(get_type(sp) == T_STRING);
+}
+
+void Y_is_pointer(int nargs)
+{
+  if (nargs != 1) YError("is_pointer takes exactly one argument");
+  PushIntValue(get_type(sp) == T_POINTER);
+}
+
+void Y_identof(int argc)
+{
+  long type;
+  if (argc != 1) YError("identof takes exactly one argument");
+  type = get_type(sp);
+  if (type == -1) YError("unexpected keyword argument");
+  PushLongValue(type);
+}
+
+static int get_type(Symbol *s)
+{
+  for (;;) {
+    if (s->ops == &dataBlockSym) {
+      DataBlock *db = s->value.db;
+      if (db->ops == &lvalueOps) {
+	return ((LValue *)db)->type.base->dataOps->typeID;
+      } else {
+        return db->ops->typeID;
+      }
+    } else if (s->ops == &referenceSym) {
+      s = &globTab[s->index];
+    } else if (s->ops == &doubleScalar) {
+      return T_DOUBLE;
+    } else if (s->ops == &longScalar) {
+      return T_LONG;
+    } else if (s->ops == &intScalar) {
+      return T_INT;
+    } else {
+      /* Must be a keyword. */
+      return -1;
+    }
+  }
+}
+
+/*--------------------------------------------------------------------------*/
+
+void Y_unref(int argc)
+{
+  if (argc != 1) YError("unref takes exactly one argument");
+  if (sp->ops == &referenceSym) {
+    /* Replace reference without augmenting the reference
+       count of the data block object if it is an array. */
+    Symbol *ref = &globTab[sp->index];
+    OpTable *ops = ref->ops;
+    if (ops == &dataBlockSym) {
+      DataBlock *db = ref->value.db;
+      if (db && db->ops->isArray) {
+	/* Replace symbol in global table by nil. */
+	ref->value.db = RefNC(&nilDB);
+	sp->value.db = db; /* no Ref */
+      } else {
+	sp->value.db = Ref(db);
+      }
+    } else {
+      sp->value = ref->value;
+    }
+    sp->ops = ops; /* change ops only AFTER value updated */
+  }
+}
+
+void Y_swap(int argc)
+{
+  SymbolValue a_val, b_val;
+  OpTable *a_ops, *b_ops;
+  volatile Symbol *a_sym, *b_sym;
+  if (argc != 2) YError("swap takes exactly 2 arguments");
+  a_sym = sp;
+  b_sym = sp-1;
+  if (a_sym->ops != &referenceSym || b_sym->ops != &referenceSym)
+    YError("arguments must be simple variable references");
+  a_sym = &globTab[a_sym->index];
+  a_ops = a_sym->ops;
+  a_val = a_sym->value;
+  a_sym->ops = &intScalar;
+  b_sym = &globTab[b_sym->index];
+  b_ops = b_sym->ops;
+  b_val = b_sym->value;
+  b_sym->ops = &intScalar;
+  b_sym->value = a_val;
+  a_sym->value = b_val;
+  Drop(2);
+  b_sym->ops = a_ops;
+  a_sym->ops = b_ops;
+}
+
 /*--------------------------------------------------------------------------*/
 
 extern VMaction DropTop;
@@ -966,6 +1236,12 @@ extern double log10(double);
 extern double sqrt(double);
 extern double ceil(double);
 extern double floor(double);
+#ifdef HAVE_ROUND
+extern double round(double);
+#endif
+#ifdef HAVE_LROUND
+extern long lround(double);
+#endif
 
 /* function either present in math library or implemented in nonc.c */
 extern double hypot(double, double);
@@ -1186,6 +1462,98 @@ static void floorZLoop(double *dst, double *src, long n)
 
 void Y_floor(int nArgs) { UnaryTemplate(nArgs, &floorLoop, &floorZLoop); }
 
+/* ----- round ----- */
+
+void Y_round(int nArgs)
+{
+  Operand op;
+  long number, i;
+  const double *src;
+  double *dst;
+  int promoteID, inPlace, errCode;
+
+  if (nArgs != 1) YError("round takes exactly one argument");
+  sp->ops->FormOperand(sp, &op);
+  if ((promoteID = op.ops->promoteID) > T_DOUBLE)
+    YError("expecting non-complex numeric argument");
+  if (promoteID < T_DOUBLE) {
+    op.ops->ToDouble(&op);
+    if (promoteID <= T_LONG) {
+      PopToD(sp - 1);
+      return;
+    }
+    inPlace = 1;
+    dst = op.value;
+  } else {
+    inPlace = 0;
+    dst = BuildResultU(&op, &doubleStruct);
+  }
+  src = op.value;
+  number = op.type.number;
+  errno = 0;
+  for (i = 0; i < number; ++i) {
+#ifdef HAVE_ROUND
+    dst[i] = round(src[i]);
+#else
+    dst[i] = floor(src[i] + 0.5);
+#endif
+  }
+  errCode = errno;
+  if (inPlace) {
+    PopToD(sp - 1);
+  } else {
+    PopToD(sp - 2);
+    Drop(1);
+  }
+  if (errCode != 0) YError("mathlib function signals error");
+}
+
+/* ----- lround ----- */
+
+void Y_lround(int nArgs)
+{
+  Operand op;
+  long number, i;
+  const double *src;
+  long *dst;
+  int promoteID, errCode;
+
+  if (nArgs != 1) YError("lround takes exactly one argument");
+  sp->ops->FormOperand(sp, &op);
+  if ((promoteID = op.ops->promoteID) > T_DOUBLE)
+    YError("expecting non-complex numeric argument");
+  if (promoteID <= T_LONG) {
+    if (promoteID < T_LONG) {
+      op.ops->ToLong(&op);
+    }
+    PopToD(sp - 1);
+    return;
+  }
+  if (promoteID < T_DOUBLE) {
+    /* FIXME: we could use lroundf and/or roundf */
+    op.ops->ToDouble(&op);
+  }
+  src = op.value;
+  dst = BuildResultU(&op, &longStruct);
+  number = op.type.number;
+  errno = 0;
+  for (i = 0; i < number; ++i) {
+#ifdef HAVE_LROUND
+    dst[i] = lround(src[i]);
+#else
+# ifdef HAVE_ROUND
+    dst[i] = (long)round(src[i]);
+# else
+    dst[i] = (long)floor(src[i] + 0.5);
+# endif
+#endif
+  }
+  errCode = errno;
+  PopToL(sp - 2);
+  Drop(1);
+  if (errCode != 0) YError("mathlib function signals error");
+}
+
 /* ----- abs ----- */
 
 typedef void AbsLooper(void *, void *, long);
