@@ -1,13 +1,14 @@
 /*
-    STD1.C
-    More Yorick built-in functions declared in std.i
-
-    See std.i for documentation on the functions defined here.
-
-    $Id: std1.c,v 1.1 1993/08/27 18:32:09 munro Exp $
+ * $Id: std1.c,v 1.3 2009/11/23 21:20:28 dhmunro Exp $
+ * More Yorick built-in functions declared in std.i
+ *
+ *  See std.i for documentation on the functions defined here.
+ */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
  */
-/*    Copyright (c) 1994.  The Regents of the University of California.
-                    All rights reserved.  */
 
 #include "ydata.h"
 #include "pstdlib.h"
@@ -17,9 +18,8 @@
 #include <time.h>
 
 extern BuiltIn Y_indgen, Y_span, Y_digitize, Y_interp, Y_integ, Y_sort,
-  Y_transpose, Y_grow, Y__, Y_strlen, Y_strpart, Y_strmatch, Y_strtok,
-  Y_timestamp, Y_timer, Y_random, Y_random_seed, Y_merge, Y_histogram,
-  Y_poly;
+  Y_transpose, Y_grow, Y__, Y_timestamp, Y_timer,
+  Y_random, Y_random_seed, Y_merge, Y_histogram, Y_poly, Y_noop;
 
 /*--------------------------------------------------------------------------*/
 
@@ -1001,197 +1001,23 @@ void Y__(int nArgs)
 
 /*--------------------------------------------------------------------------*/
 
-void Y_strlen(int nArgs)
-{
-  long i, n, *lens;
-  Dimension *dims;
-  char **input;
-
-  if (nArgs != 1) YError("strlen takes exactly one argument");
-
-  input= YGet_Q(sp, 0, &dims);
-  n= TotalNumber(dims);
-
-  if (dims) {
-    Array *result= PushDataBlock(NewArray(&longStruct, dims));
-    lens= result->value.l;
-  } else {
-    PushLongValue(0L);
-    lens= &sp->value.l;
-  }
-
-  for (i=0 ; i<n ; i++) {
-    if (input[i]) lens[i]= strlen(input[i]);
-    else lens[i]= 0;
-  }
-}
-
-void Y_strpart(int nArgs)
-{
-  Operand op;
-  Range *range;
-  long min, max, i, n, len, mn, mx;
-  int maxNil;
-  Dimension *dims;
-  char **input, **output;
-  Array *result;
-
-  if (nArgs != 2) YError("strpart takes exactly two arguments");
-
-  sp->ops->FormOperand(sp, &op);
-  range= op.value;
-  if (op.ops!=&rangeOps || range->rf || range->inc!=1 ||
-      range->nilFlags&(R_PSEUDO|R_RUBBER|R_NULLER))
-    YError("bad 2nd argument to strpart");
-  min= range->nilFlags&R_MINNIL? 0 : range->min-1L;
-  maxNil= range->nilFlags&R_MAXNIL;
-  max= range->max-1L;
-
-  input= YGet_Q(sp-1, 0, &dims);
-  n= TotalNumber(dims);
-
-  result= PushDataBlock(NewArray(&stringStruct, dims));
-  output= result->value.q;
-
-  for (i=0 ; i<n ; i++) {
-    if (input[i]) {
-      len= strlen(input[i]);
-      mn= min<0? len+min : min;
-      if (maxNil) mx= len-1;
-      else mx= max<0? len+max : max;
-      if (mn<0) mn= 0;
-      if (mx<0) mx= -1;
-      if (mx>=len) mx= len-1;
-      if (mn<len && mx>=mn) output[i]= p_strncat(0, input[i]+mn, mx-mn+1);
-      else output[i]= p_strncat(0, "", 1);
-    } else {
-      output[i]= 0;
-    }
-  }
-}
-
-void Y_strmatch(int nArgs)
-{
-  long i, n;
-  long caseFold;
-  char *pattern, *s, *p;
-  Dimension *dims;
-  char **input;
-  int *output, uorl, a, z;
-
-  if (nArgs!=2 && nArgs!=3)
-    YError("strmatch takes exactly two or three arguments");
-
-  if (nArgs==3) { caseFold= YGetInteger(sp); Drop(1); }
-  else caseFold= 0;
-  uorl= ('A'^'a');   /* probably no safer than '\040' */
-  a= ('a'|uorl);     /* probably no safer than 'a' */
-  z= ('z'|uorl);     /* probably no safer than 'z' */
-
-  pattern= YGetString(sp);
-
-  input= YGet_Q(sp-1, 0, &dims);
-  n= TotalNumber(dims);
-
-  if (dims) {
-    Array *result= PushDataBlock(NewArray(&intStruct, dims));
-    output= result->value.i;
-  } else {
-    PushIntValue(0);
-    output= &sp->value.i;
-  }
-
-  /* note that pattern=="" always matches, pattern==0 never matches */
-  for (i=0 ; i<n ; i++) {
-    if (pattern && input[i]) {
-      s= input[i]; p= pattern;
-      if (caseFold) {
-        while (*s) {
-          while (*s && !(*s==*p || ((*s|uorl)==(*p|uorl) &&
-                                    (*s|uorl)>=a && (*s|uorl)<=z))) s++;
-          if (!*s) break;
-          do { s++; p++; } while (*s &&
-                                  (*s==*p || ((*s|uorl)==(*p|uorl) &&
-                                              (*s|uorl)>=a && (*s|uorl)<=z)));
-          if (!*p) break;
-          p= pattern;
-        }
-      } else {
-        while (*s) {
-          while (*s && *s!=*p) s++;
-          if (!*s) break;
-          do { s++; p++; } while (*s && *s==*p);
-          if (!*p) break;
-          p= pattern;
-        }
-      }
-      output[i]= (*p=='\0');
-    } else {
-      output[i]= 0;
-    }
-  }
-}
-
-static char *current;
-
-void Y_strtok(int nArgs)
-{
-  Symbol *s= sp-nArgs+1;
-  long i, n;
-  char **input, **output, *delimit;
-  Array *result;
-  Dimension *dims;
-
-  if (nArgs!=1 && nArgs!=2)
-    YError("strtok takes exactly one or two arguments");
-
-  if (nArgs==2) {
-    delimit= YGetString(sp);
-    if (!delimit) delimit= " \t\n";
-  } else {
-    delimit= " \t\n";
-  }
-
-  dims= tmpDims;
-  tmpDims= 0;
-  FreeDimension(dims);
-
-  input= YGet_Q(s, 0, &dims);
-  n= TotalNumber(dims);
-
-  tmpDims= NewDimension(2L, 1L, (Dimension *)0);
-  tmpDims= CopyDims(dims, tmpDims, 1);
-
-  result= PushDataBlock(NewArray(&stringStruct, tmpDims));
-  output= result->value.q;
-
-  for (i=0 ; i<n ; i++) {
-    if (input[i]) {
-      p_free(current);  current= 0;
-      current= p_strcpy(input[i]);
-      output[2*i]= p_strcpy(strtok(current, delimit));
-      output[2*i+1]= p_strcpy(strtok((char *)0, ""));
-    } else {
-      output[2*i]= output[2*i+1]= 0;
-    }
-  }
-  p_free(current);  current= 0;
-}
-
-/*--------------------------------------------------------------------------*/
-
 extern char *Ytimestamp(void);  /* 25 character return */
 
 void
-Y_timestamp(int nArgs)
+Y_timestamp(int argc)
 {
-  time_t n_time;
-  Array *array;
-  if (nArgs!=1 || YNotNil(sp))
-    YError("timestamp takes exactly one nil argument");
-  array = PushDataBlock(NewArray(&stringStruct, (Dimension *)0));
-  n_time = time((void *)0);
-  array->value.q[0] = p_strcpy(strtok(ctime(&n_time), "\n"));
+  time_t n_time = time((void *)0);
+  char *time = ctime(&n_time);
+  long index = yget_ref(0);
+  if (argc != 1) y_error("timestamp takes exactly one argument");
+  if (index >= 0) {
+    ypush_long((long)n_time);
+    yput_global(index, 0);
+  }
+  if (!yarg_subroutine()) {
+    char **q = ypush_q(0);
+    q[0] = p_strcpy(strtok(time, "\n"));
+  }
 }
 
 static double y_wall0 = 0.;
@@ -1594,3 +1420,12 @@ void Y_poly(int nArgs)
 }
 
 /*--------------------------------------------------------------------------*/
+
+void
+Y_noop(int argc)
+{
+  if (argc && sp->ops==&referenceSym) ReplaceRef(sp);
+  return;
+}
+
+/*--------------------------------------------------------------------------*/
