@@ -1,13 +1,14 @@
 /*
-    STD2.C
-    Define standard Yorick built-in functions for binary I/O
-
-    See std.i for documentation on the interface functions defined here.
-
-    $Id: std2.c,v 1.1 1993/08/27 18:32:09 munro Exp $
+ * $Id: std2.c,v 1.1.1.1 2005/09/18 22:04:07 dhmunro Exp $
+ * Define standard Yorick built-in functions for binary I/O
+ *
+ *  See std.i for documentation on the interface functions defined here.
+ */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
  */
-/*    Copyright (c) 1994.  The Regents of the University of California.
-                    All rights reserved.  */
 
 #include "ydata.h"
 #include "yio.h"
@@ -294,7 +295,7 @@ static void GetTNworker(int nArgs, int cycs)
   Dimension *dims;
   if (nArgs!=1) YError("get_times/get_ncycs takes exactly one argument");
 
-  file= YGetFile(sp);
+  file= yarg_file(0);
   history= file->history;
 
   if (history) {
@@ -405,7 +406,7 @@ void Y__jc(int nArgs)
   int noRecord;
   if (nArgs!=2) YError("jc takes exactly two arguments");
 
-  file= YGetFile(sp-1);
+  file= yarg_file(1);
   history= file->history;
   if (!history)
     YError("binary file passed to jc has no history records");
@@ -426,7 +427,7 @@ void Y__jr(int nArgs)
   int amSubroutine= CalledAsSubroutine();
   if (nArgs!=2) YError("jr takes exactly two arguments");
 
-  file= YGetFile(sp-1);
+  file= yarg_file(1);
   history= file->history;
   if (!history) {
     if (amSubroutine)
@@ -455,7 +456,7 @@ void Y_get_vars(int nArgs)
   Dimension *dims;
   if (nArgs!=1) YError("get_vars takes exactly one argument");
 
-  file= YGetFile(sp);
+  file= yarg_file(0);
 
   child= file->history? file->history->child : 0;
 
@@ -575,7 +576,7 @@ void Y_get_addrs(int nArgs)
   Dimension *dims;
   if (nArgs!=1) YError("get_addrs takes exactly one argument");
 
-  file= YGetFile(sp);
+  file= yarg_file(0);
 
   history= file->history;
   child= history? history->child : 0;
@@ -635,7 +636,7 @@ void Y_set_filesize(int nArgs)
   long size;
   if (nArgs!=2) YError("set_filesize takes exactly two arguments");
 
-  file= YGetFile(sp-1);
+  file= yarg_file(1);
   history= file->history;
   if (!history)
     YError("binary file passed to set_filesize has no history records");
@@ -651,7 +652,7 @@ void Y_set_blocksize(int nArgs)
   long size, nbytes;
   if (nArgs!=2) YError("set_block takes exactly two arguments");
 
-  file= YGetFile(sp-1);
+  file= yarg_file(1);
 
   nbytes= YGetInteger(sp);
   size= 4096;
@@ -812,7 +813,7 @@ void Y__read(int nArgs)
   StructDef *base;
   if (nArgs!=3) YError("_read takes exactly three arguments");
 
-  file= YGetFile(sp-2);
+  file= yarg_file(2);
   if (file->history) file= file->history->child;
 
   address= YGetInteger(sp-1);
@@ -1020,7 +1021,7 @@ void Y_get_primitives(int nArgs)
   int i, j;
   if (nArgs!=1) YError("get_primitives requires exactly one argument");
 
-  file = YGetFile(sp);
+  file = yarg_file(0);
   dims = tmpDims;
   tmpDims = 0;
   FreeDimension(dims);
@@ -1089,7 +1090,7 @@ void Y_data_align(int nArgs)
   int alignment;
   if (nArgs!=2) YError("data_align takes exactly two arguments");
 
-  file= YGetFile(sp-1);
+  file= yarg_file(1);
   alignment= (int)YGetInteger(sp);
 
   file->dataAlign= alignment>0? alignment : 0;
@@ -1102,7 +1103,7 @@ void Y_struct_align(int nArgs)
   int alignment;
   if (nArgs!=2) YError("struct_align takes exactly two arguments");
 
-  file= YGetFile(sp-1);
+  file= yarg_file(1);
   alignment= (int)YGetInteger(sp);
 
   file->structAlign= (alignment!=0)? alignment : yStructAlign;
@@ -1198,7 +1199,7 @@ void Y__not_pdb(int nArgs)
   int familyOK, notOK;
   if (nArgs!=2) YError("_not_pdb takes exactly two arguments");
 
-  file= YGetFile(sp-1);
+  file= yarg_file(1);
   familyOK= (int)YGetInteger(sp);
 
   if (!pdb_open) pdb_open= Globalize("yPDBopen", 0L);
@@ -1229,7 +1230,7 @@ void Y__init_pdb(int nArgs)
   int close102;
   if (nArgs!=2) YError("_init_pdb takes exactly two arguments");
 
-  file= YGetFile(sp-1);
+  file= yarg_file(1);
   close102= ((int)YGetInteger(sp)) & 017;
 
   if (YinitPDB(file, close102))
@@ -1243,7 +1244,7 @@ void Y__set_pdb(int nArgs)
   int close102;
   if (nArgs!=2) YError("_set_pdb takes exactly two arguments");
 
-  file= YGetFile(sp-1);
+  file= yarg_file(1);
   close102= ((int)YGetInteger(sp)) & 017;
 
   if (file->history) {
@@ -1260,7 +1261,7 @@ void Y__init_clog(int nArgs)
   IOStream *file;
   if (nArgs!=1) YError("_init_clog takes exactly one argument");
 
-  file= YGetFile(sp);
+  file= yarg_file(0);
 
   if (file->history) {
     file->history->parent->CloseHook=
@@ -1277,7 +1278,7 @@ void Y_dump_clog(int nArgs)
   char *name;
   if (nArgs!=2) YError("dump_clog takes exactly two arguments");
 
-  file= YGetFile(sp-1);
+  file= yarg_file(1);
   name= YGetString(sp);
 
   if (DumpClogFile(file, name))
@@ -1291,8 +1292,8 @@ void Y_read_clog(int nArgs)
   char *name;
   if (nArgs!=2) YError("read_clog takes exactly two arguments");
 
-  file= YGetFile(sp-1);
-  name= YGetString(sp);
+  file= yarg_file(1);
+  name= yarg_sq(0);
 
   if (ReadClogFile(file, name))
     YError("read_clog failed -- unable to read input file");
@@ -1370,16 +1371,7 @@ struct y_tmpobj {
 
 static void y_zap_tmpobj(void *to);
 
-extern PromoteOp PromXX;
-extern UnaryOp ToAnyX, NegateX, ComplementX, NotX, TrueX;
-extern BinaryOp AddX, SubtractX, MultiplyX, DivideX, ModuloX, PowerX;
-extern BinaryOp EqualX, NotEqualX, GreaterX, GreaterEQX;
-extern BinaryOp ShiftLX, ShiftRX, OrX, AndX, XorX;
-extern BinaryOp AssignX, MatMultX;
-extern UnaryOp EvalX, SetupX, PrintX;
-extern MemberOp GetMemberX;
-
-static Operations yo_tmpobj = {
+Operations yo_tmpobj = {
   &y_zap_tmpobj, T_OPAQUE, 0, T_STRING, "tmpobj",
   {&PromXX, &PromXX, &PromXX, &PromXX, &PromXX, &PromXX, &PromXX, &PromXX},
   &ToAnyX, &ToAnyX, &ToAnyX, &ToAnyX, &ToAnyX, &ToAnyX, &ToAnyX,
