@@ -1,5 +1,5 @@
 /*
- * $Id: task.c,v 1.7 2007/02/24 23:14:41 dhmunro Exp $
+ * $Id: task.c,v 1.15 2010/04/16 05:23:43 dhmunro Exp $
  * Implement Yorick virtual machine.
  */
 /* Copyright (c) 2005, The Regents of the University of California.
@@ -33,8 +33,9 @@ extern int YpParse(void *func);  /* argument non-zero for YpReparse */
 /*--------------------------------------------------------------------------*/
 
 extern BuiltIn Y_quit, Y_include, Y_require, Y_help, Y_exit, Y_error, Y_batch;
+extern BuiltIn Y_current_include, Y_get_includes;
 extern BuiltIn Y_plug_in, Y_plug_dir, Y_maybe_prompt, Y_suspend, Y_resume;
-extern BuiltIn Y_after;
+extern BuiltIn Y_after, Y_include1, Y_vopen, Y_vclose;
 
 extern void YRun(void);
 extern void YHalt(void);
@@ -56,7 +57,7 @@ int yAutoDebug= 0;
 int yDebugLevel= 0;
 
 /* most recent error message was built in yErrorMsg */
-char yErrorMsg[192];
+char yErrorMsg[192+12];
 char yWarningMsg[192];
 
 static int inYError= 0;
@@ -525,32 +526,428 @@ void IncludeNow(void)
   if (ym_state&Y_QUITTING) YHalt();
 }
 
+static char *y_include_arg(p_file **file);
+
 void
 Y_include(int nArgs)
 {
-  long now;
+  long now = 0;
   char *name;
+  p_file *file = 0;
   if (nArgs!=1 && nArgs!=2)
     YError("include function takes exactly one or two arguments");
-  now = (nArgs>1)? YGetInteger(sp) : 0;
-  name = YGetString(sp-nArgs+1);
+  if (nArgs > 1) {
+    now = YGetInteger(sp);
+    Drop(1);
+  }
+  name = y_include_arg(&file);
   if (name[0]) {  /* name=="" special hack cleans out pending includes */
     if (now >= 0) {
-      if (!YpPushInclude(name)) {
+      if (file) {
+        y_push_include(file, name);
+      } else if (!YpPushInclude(name)) {
         if (!(now&2))
           YError("missing include file specified in include function");
         now = 0;
       }
     } else {
+      if (file) YError("include: now<0 only possible with filename argument");
       YpPush(name);          /* defer until all pending input parsed */
     }
   }
-  Drop(nArgs);
+  Drop(1);
   if (now>0) IncludeNow(); /* parse and maybe execute file to be included
                             * -- without now, this won't happen until the
                             * next line is parsed naturally */
 }
 
+void
+Y_include1(int nArgs)
+{
+  int i0 = nYpIncludes;
+  int t0 = nTasks;
+  char *name;
+  p_file *file = 0;
+  if (nArgs != 1) YError("include1 function takes exactly one argument");
+  name = y_include_arg(&file);
+  if (file) y_push_include(file, name);
+  else if (!YpPushInclude(name))
+    YError("missing include file specified in include1 function");
+  Drop(nArgs);
+  if (ym_state & Y_SUSPENDED)
+    YError("include1 called while suspended waiting for event");
+  while (nTasks<=t0 && (nYpIncludes>i0 || (nYpIncludes==i0 &&
+         ypIncludes[i0-1].file))) YpParse((void *)0);
+  if (nTasks == t0+1) {
+    (sp+1)->ops = &dataBlockSym;
+    (sp+1)->value.db = (DataBlock *)tasks[--nTasks];  /* use owned by stack */
+    sp++;
+  } else if (nTasks <= t0) {
+    PushDataBlock(RefNC(&nilDB));
+  } else {
+    YError("include1 created more than one task (impossible?)");
+  }
+}
+
+static p_file *ynew_vopen(Array *array, int binary);
+static char *y_vopen_name = "(vopen file)";
+
+static char *
+y_include_arg(p_file **file)
+{
+  Operand op;
+  char *name = y_vopen_name;
+  if (!sp->ops) YError("unexpected keyword argument in include or include1");
+  sp->ops->FormOperand(sp, &op);
+  if (op.ops->typeID == T_STRING) {
+    if (!op.type.dims) {
+      char **q = op.value;
+      if (!q[0]) YError("string(0) filename to include or include1");
+      name = q[0];
+    } else {
+      *file = ynew_vopen((Array *)sp->value.db, 0);
+      name = y_vopen_name;
+    }
+  } else if (op.ops->typeID == T_CHAR) {
+    *file = ynew_vopen((Array *)sp->value.db, 0);
+    name = y_vopen_name;
+  } else if (op.ops == &textOps) {
+    /* first few members of TextStream same as IOStream */
+    name = ((IOStream *)op.value)->fullname;
+  } else {
+    if (op.ops == &streamOps)
+      YError("include or include1 cannot accept binary file handle");
+    YError("include or include1 cannot convert argument to text file handle");
+  }
+  return name;
+}
+
+/* ----- begin vopen implementation ----- */
+
+static unsigned long yv_fsize(p_file *file);
+static unsigned long yv_ftell(p_file *file);
+static int yv_fseek(p_file *file, unsigned long addr);
+
+static char *yv_fgets(p_file *file, char *buf, int buflen);
+static int yv_fputs(p_file *file, const char *buf);
+static unsigned long yv_fread(p_file *file,
+                              void *buf, unsigned long nbytes);
+static unsigned long yv_fwrite(p_file *file,
+                               const void *buf, unsigned long nbytes);
+
+static int yv_feof(p_file *file);
+static int yv_ferror(p_file *file);
+static int yv_fflush(p_file *file);
+static int yv_fclose(p_file *file);
+
+static p_file_ops y_vopen_ops = {
+  &yv_fsize, &yv_ftell, &yv_fseek,
+  &yv_fgets, &yv_fputs, &yv_fread, &yv_fwrite,
+  &yv_feof, &yv_ferror, &yv_fflush, &yv_fclose };
+
+typedef struct y_vopen_t y_vopen_t;
+struct y_vopen_t {
+  p_file_ops *ops;
+  Array *array;
+  long addr, maxaddr;
+  int binary;
+};
+
+static p_file *
+ynew_vopen(Array *array, int binary)
+{
+  y_vopen_t *file = p_malloc(sizeof(y_vopen_t));
+  file->ops = &y_vopen_ops;
+  file->array = Ref(array);
+  file->addr = 0;
+  file->maxaddr = (binary&2)? 0 : ((y_vopen_t *)file)->array->type.number;
+  file->binary = binary;
+  return (p_file *)file;
+}
+
+void *
+y_vopen_file(void *stream)
+{
+  y_vopen_t *file = stream;
+  return (file->ops==&y_vopen_ops)? file->array : 0;
+}
+
+void
+Y_vopen(int argc)
+{
+  Operand op;
+  p_file *file = 0;
+  int binary = 0, wrt = 0;
+  if (argc == 2) {
+    if (!sp[-1].ops) YError("vopen: unexpected keyword argument");
+    binary = (YGetInteger(sp) != 0);
+    Drop(1);
+  } else if (argc != 1) {
+    YError("vopen takes one or two arguments");
+  }
+  sp->ops->FormOperand(sp, &op);
+  if (op.ops->typeID == T_VOID) {
+    Dimension *tmp = tmpDims;
+    tmpDims = 0;
+    FreeDimension(tmp);
+    tmpDims = NewDimension(binary? 16384L : 1024L, 1L, (Dimension *)0);
+    Drop(1);
+    if (binary) PushDataBlock(NewArray(&charStruct, tmpDims));
+    else PushDataBlock(NewArray(&stringStruct, tmpDims));
+    sp->ops->FormOperand(sp, &op);
+    binary |= (wrt = 2);
+  }
+  if (op.ops->typeID!=T_STRING && op.ops->typeID!=T_CHAR)
+    YError("vopen argument must be string or char array");
+  file = ynew_vopen((Array *)sp->value.db, binary);
+  if (binary&1)
+    PushDataBlock(NewIOStream(p_strcpy(y_vopen_name), file, 9|wrt));
+  else
+    PushDataBlock(NewTextStream(p_strcpy(y_vopen_name), file, 1|wrt, 0L, 0L));
+}
+
+void
+Y_vclose(int argc)
+{
+  long index = -1;
+  Operand op;
+  if (argc != 1) YError("vclose takes exactly one argument");
+  if (sp->ops == &referenceSym) index = sp->index;
+  sp->ops->FormOperand(sp, &op);
+  if (op.ops==&textOps || op.ops==&streamOps) {
+    IOStream *ios = op.value;  /* first few members match TextStream */
+    y_vopen_t *file = ios->stream;
+    if (file && file->ops==&y_vopen_ops) {
+      long len;
+      len = file->maxaddr;
+      if (!len) {
+        PushDataBlock(RefNC(&nilDB));
+      } else {
+        if (file->binary & 2) {
+          if (file->binary & 1) {
+            if (ios->CloseHook) {
+              ios->CloseHook(ios);
+              ios->CloseHook = 0;
+              len = file->maxaddr;
+            }
+          }
+          if (file->array->type.number > len) {
+            /* shrink array to elements actually used */
+            file->array->type.number =
+              file->array->type.dims->number = len;
+            if (!(file->binary & 1)) len *= sizeof(char*);
+            len += (char *)file->array->value.q - (char *)file->array;
+            file->array = p_realloc(file->array, len);
+          }
+        }
+        PushDataBlock(Ref(file->array));
+        if ((file->binary & 3) == 3) {
+          ios->ioOps->Close(ios);
+          ios->stream = 0;
+        }
+      }
+      if (index >= 0) {
+        /* set reference argument to nil */
+        Symbol *s = &globTab[index];
+        if (s->ops==&dataBlockSym && s->value.db==op.value) {
+          s->ops = &intScalar;
+          Unref(s->value.db);
+          s->value.db = RefNC(&nilDB);
+          s->ops = &dataBlockSym;
+        }
+      }
+      return;
+    }
+  }
+  YError("vclose: already closed, not vopen handle, or not a file handle");
+}
+
+static unsigned long
+yv_fsize(p_file *file)
+{
+  if (((y_vopen_t *)file)->binary & 2) return ((y_vopen_t *)file)->maxaddr;
+  else return ((y_vopen_t *)file)->array->type.number;
+}
+
+static unsigned long
+yv_ftell(p_file *file)
+{
+  return ((y_vopen_t *)file)->addr;
+}
+
+static int
+yv_fseek(p_file *file, unsigned long addr)
+{
+  long len = ((y_vopen_t *)file)->array->type.number;
+  if (((y_vopen_t *)file)->binary & 2) {
+    if (((y_vopen_t *)file)->binary & 1) {
+      if (addr > len) {
+        long j, n = 2*((y_vopen_t *)file)->array->type.number;
+        long nhead = (char *)((y_vopen_t *)file)->array->value.c -
+          (char *)((y_vopen_t *)file)->array;
+        while (n < addr) n += n;
+        ((y_vopen_t *)file)->array = p_realloc(((y_vopen_t *)file)->array,
+                                               nhead+n);
+        for (j=len ; j<n ; j++) ((y_vopen_t *)file)->array->value.c[j] = '\0';
+        ((y_vopen_t *)file)->array->type.number =
+          ((y_vopen_t *)file)->array->type.dims->number = n;
+      }
+      len = addr;
+    } else {
+      len = ((y_vopen_t *)file)->maxaddr;
+    }
+  }
+  if (addr<0 || addr>len) return -1;
+  ((y_vopen_t *)file)->addr = addr;
+  if (addr > ((y_vopen_t *)file)->maxaddr)
+    ((y_vopen_t *)file)->maxaddr = addr;
+  return 0;
+}
+
+static char *
+yv_fgets(p_file *file, char *buf, int buflen)
+{
+  int strng = (((y_vopen_t *)file)->array->ops->typeID == T_STRING);
+  char *txt, c='\0';
+  long jeof;
+  int i, j;
+  if (!strng) {
+    txt = ((y_vopen_t *)file)->array->value.c + ((y_vopen_t *)file)->addr;
+    jeof = ((y_vopen_t *)file)->array->type.number - ((y_vopen_t *)file)->addr;
+  } else {
+    txt = ((y_vopen_t *)file)->array->value.q[((y_vopen_t *)file)->addr];
+    jeof = 0L;
+  }
+  if (buflen <= 0) return 0;
+  for (i=j=0 ; i<buflen-1 && c!='\n' ; i++,j++) {
+    if (!strng && j>=jeof) break;
+    if (!txt || !txt[j]) {
+      c = '\n';
+    } else if (txt[j] == '\r') {
+      if (j<jeof-1 && txt[j+1]=='\n') j++;
+      c = '\n';
+    } else {
+      c = txt[j];
+    }
+    buf[i] = c;
+  }
+  buf[i] = '\0';
+  if (!strng) ((y_vopen_t *)file)->addr += j;
+  else ((y_vopen_t *)file)->addr++;
+  return buf;
+}
+
+static int
+yv_fputs(p_file *file, const char *buf)
+{
+  if (!buf) return 0;
+  if (((y_vopen_t *)file)->binary == 2) {
+    long n, addr = ((y_vopen_t *)file)->addr;
+    long len = ((y_vopen_t *)file)->array->type.number;
+    long nhead = (char *)((y_vopen_t *)file)->array->value.q -
+      (char *)((y_vopen_t *)file)->array;
+    char *line;
+    for (;;) {
+      if (addr == len) {
+        n = nhead + (len+len)*sizeof(char*);
+        ((y_vopen_t *)file)->array = p_realloc(((y_vopen_t *)file)->array, n);
+        for (n=0 ; n<len ; n++) ((y_vopen_t *)file)->array->value.q[len+n] = 0;
+        ((y_vopen_t *)file)->array->type.number =
+          ((y_vopen_t *)file)->array->type.dims->number = len + len;
+      }
+      for (n=0 ; buf[n] && buf[n]!='\n' ; n++);
+      line = ((y_vopen_t *)file)->array->value.q[addr];
+      if (n) {
+        ((y_vopen_t *)file)->array->value.q[addr] = p_strncat(line, buf, n);
+        if (line) p_free(line);
+      } else if (!line) {
+        ((y_vopen_t *)file)->array->value.q[addr] = p_strcpy("");
+      }
+      if (buf[n] == '\n') addr++, n++;
+      if (!buf[n]) break;
+      buf += n;
+    }
+    ((y_vopen_t *)file)->addr = addr;
+    if (addr > ((y_vopen_t *)file)->maxaddr)
+      ((y_vopen_t *)file)->maxaddr = addr;
+  } else {
+    YError("p_fputs to binary or read-only vopen file handle");
+  }
+  return 0;
+}
+
+static unsigned long
+yv_fread(p_file *file, void *buf, unsigned long nbytes)
+{
+  int strng = (((y_vopen_t *)file)->array->ops->typeID == T_STRING);
+  char *cbuf=buf, *txt=0;
+  unsigned long j, jeof=0;
+  if (!strng) {
+    txt = ((y_vopen_t *)file)->array->value.c + ((y_vopen_t *)file)->addr;
+    jeof = ((y_vopen_t *)file)->array->type.number - ((y_vopen_t *)file)->addr;
+  } else {
+    YError("p_fread from text vopen file handle");
+  }
+  if (!nbytes) return 0L;
+  for (j=0 ; j<nbytes ; j++) {
+    if (!strng && j>=jeof) break;
+    cbuf[j] = txt[j];
+    if (strng && !txt[j]) break;
+  }
+  if (!strng) ((y_vopen_t *)file)->addr += j;
+  else ((y_vopen_t *)file)->addr++;
+  return j;
+}
+
+static unsigned long
+yv_fwrite(p_file *file, const void *buf, unsigned long nbytes)
+{
+  if (((y_vopen_t *)file)->binary == 3) {
+    long len = ((y_vopen_t *)file)->array->type.number;
+    long i = ((y_vopen_t *)file)->addr;
+    if (i+nbytes > len) {
+      /* double array size if at eof */
+      long j, n = len + len;
+      long nhead = (char *)((y_vopen_t *)file)->array->value.c -
+        (char *)((y_vopen_t *)file)->array;
+      while (n < i+nbytes) n += n;
+      ((y_vopen_t *)file)->array = p_realloc(((y_vopen_t *)file)->array,
+                                             nhead + n);
+      for (j=len ; j<n ; j++) ((y_vopen_t *)file)->array->value.c[j] = '\0';
+      ((y_vopen_t *)file)->array->type.number =
+        ((y_vopen_t *)file)->array->type.dims->number = n;
+    }
+    if (nbytes) memcpy(((y_vopen_t *)file)->array->value.c+i, buf, nbytes);
+    ((y_vopen_t *)file)->addr = (i += nbytes);
+    if (i > ((y_vopen_t *)file)->maxaddr) ((y_vopen_t *)file)->maxaddr = i;
+  } else {
+    YError("p_fwrite to text or read-only vopen file handle");
+  }
+  return nbytes;
+}
+
+static int
+yv_feof(p_file *file)
+{
+  return (((y_vopen_t *)file)->addr
+          >= ((y_vopen_t *)file)->array->type.number);
+}
+
+static int yv_ferror(p_file *file) { return 0; }
+static int yv_fflush(p_file *file) { return 0; }
+
+static int
+yv_fclose(p_file *file)
+{
+  Array *array = ((y_vopen_t *)file)->array;
+  ((y_vopen_t *)file)->array = 0;
+  Unref(array);
+  p_free(file);
+  return 0;
+}
+
+/* ----- end vopen implementation ----- */
+
 static char **yplug_path = 0;
 
 void
@@ -643,7 +1040,7 @@ y_pkg_add(y_pkg_t *init)
   if (!y_pkg_list)
     y_pkg_list = p_malloc(sizeof(struct y_package_t)*8);
   else if (!(y_npkg & 7))
-    y_pkg_list = p_malloc(sizeof(struct y_package_t)*(y_npkg+8));
+    y_pkg_list = p_realloc(y_pkg_list, sizeof(struct y_package_t)*(y_npkg+8));
   y_pkg_list[y_npkg].name = pkgname;
   y_pkg_list[y_npkg].init = init;
   y_pkg_list[y_npkg].ifiles = ifiles;
@@ -813,9 +1210,49 @@ void Y_require(int nArgs)
     IncludeNow();   /* parse and maybe execute file to be included */
 }
 
+void Y_current_include(int argc)
+{
+  if (argc != 1 || YNotNil(sp))
+    y_error("current_include takes exactly one nil argument");
+  if (nYpIncludes > 0 && ypIncludes[nYpIncludes-1].filename != NULL) {
+    *ypush_q(NULL) = p_strcpy(ypIncludes[nYpIncludes-1].filename);
+  } else {
+    ypush_nil();
+  }
+}
+
+void Y_get_includes(int argc)
+{
+  if (argc != 1 || YNotNil(sp))
+    YError("get_includes takes exactly one nil argument");
+  if (sourceTab.nItems > 0) {
+    long i, n;
+    long dims[2];
+    char **s;
+    dims[0] = 1L;
+    dims[1] = (n = sourceTab.nItems);
+    s = ypush_q(dims);
+    for (i = 0; i < n; ++i) {
+      s[i] = p_strcpy(sourceTab.names[i]);
+    }
+  } else {
+    ypush_nil();
+  }
+}
+
 /*--------------------------------------------------------------------------*/
 
 static int findingFunc= 0;
+/* error handling hacks (for mpy) set up with set_idler
+ * bit
+ *  1  do not print error messages (like .SYNC.)
+ *  2  include [pc] after func name in error messages
+ *  4  call after_error in dbug mode (rather than full stack reset)
+ *     - after_error responsible for calling dbexit
+ *  8  reserved for use by y_errhook
+ */
+PLUG_API int yerror_flags;
+int yerror_flags = 0;
 
 Function *FuncContaining(Instruction *pc)
 {
@@ -845,8 +1282,9 @@ Function *FuncContaining(Instruction *pc)
   if (findingFunc) {
     /* may get here after a disaster causing an interrupt above, as well
        as after scanning from a garbled initial pc */
+    int no_pf = ((yerror_flags&1) != 0);
     findingFunc= 0;
-    YputsErr("(BUG) lost function produced following error:");
+    if (!no_pf) YputsErr("(BUG) lost function produced following error:");
   }
   return func;
 }
@@ -911,6 +1349,9 @@ Instruction *yErrorPC= 0;   /* for dbup function in debug.c */
 static void yset_catchmsg(char *tmsg);
 static long after_index = -1;
 
+PLUG_API int (*y_errhook)(const char *msg, long *after);
+int (*y_errhook)(const char *msg, long *after) = 0;
+
 void
 YError(const char *msg)
 {
@@ -920,11 +1361,11 @@ YError(const char *msg)
   Function *func;
   char *name;
   DebugBlk *dbg;
-  Instruction *pcUp= yErrorPC;
+  Instruction *pcUp=yErrorPC, *pcue;
   int category;
   int no_abort = y_do_not_abort;
-  int no_print = 0;
-  char tmsg[144];
+  int no_print = 0, no_pf = ((yerror_flags&1) != 0);
+  int no_reset = ((yerror_flags&4) != 0);
 
   int recursing= inYError;
   inYError++;
@@ -938,7 +1379,7 @@ YError(const char *msg)
   ym_dbenter = 0;
 
   if (recursing>8 || yImpossible>8) {
-    YputsErr("****FATAL**** YError looping -- quitting now");
+    if (!no_pf) YputsErr("****FATAL**** YError looping -- quitting now");
     ym_state|= Y_QUITTING;
     ym_fatal = 3;
     if (!no_abort) p_abort();
@@ -956,7 +1397,7 @@ YError(const char *msg)
     return;
   } else if (caughtTask) {
     caughtTask= 0;
-    YputsErr("****OOPS**** error on read resume or throw/catch");
+    if (!no_pf) YputsErr("****OOPS**** error on read resume or throw/catch");
   }
 
   if (y_idler_function) {
@@ -972,19 +1413,24 @@ YError(const char *msg)
     /* if after_error function present, make it the idler */
     y_idler_function = (Function *)Ref(globTab[after_index].value.db);
     y_idler_fault = 1;
-    if (msg) strncpy(tmsg, msg, 140), tmsg[140] = '\0';
-    else tmsg[0] = '\0';
   } else {
     y_idler_fault = 0;
-    tmsg[0] = '\0';
   }
 
   /* this is a nasty hack for mpy and after_error */
-  no_print = (msg && !strcmp(msg, ".SYNC."));
+  no_print = no_pf || (msg && !strcmp(msg, ".SYNC."));
 
-  func= (((ym_state&Y_RUNNING)||no_abort) && !recursing &&
-         pcUp!=&taskCode[2])? FuncContaining(pcDebug) : 0;
-  name= func? globalTable.names[func->code[0].index] : "VM idle or lost";
+  for (;;) {
+    func = (((ym_state&Y_RUNNING)||no_abort) && !recursing &&
+            pcUp!=&taskCode[2])? FuncContaining(pcDebug) : 0;
+    if (!func || !func->errup) break;
+    ClearStack();
+    pcue = AbortReturn();
+    if (!pcue || pcue==&taskCode[2]) break;
+    pcDebug = pc = pcue;
+    func = 0;
+  }
+  name = func? globalTable.names[func->code[0].index] : "VM idle or lost";
 
   /* Clear out include stack, but remember current include file name.
      If the error happened while executing a main program which came from
@@ -1010,7 +1456,7 @@ YError(const char *msg)
     p_qclear();
     ClearTasks();
   } else if (nTasks) {
-    YputsErr("WARNING unable to free task pointers in YError");
+    if (!no_pf) YputsErr("WARNING unable to free task pointers in YError");
     nTasks = 0;
   }
 
@@ -1024,22 +1470,43 @@ YError(const char *msg)
   else
     strcpy(yErrorMsg, "Up to (");
   strncat(yErrorMsg, name, 40);
+  if (func && (yerror_flags&2)!=0) {
+    char relpc[32];
+    sprintf(relpc, "[%ld]", (long)(pcDebug-func->code));
+    strncat(yErrorMsg, relpc, 12);
+  }
   strcat(yErrorMsg, ") ");
   if (!pcUp || recursing)
     strncat(yErrorMsg, msg, 140);
+  if (y_errhook) {
+    long index;
+    int hook = y_errhook(yErrorMsg, &index);
+    if (no_print != no_pf) no_pf = (hook & 1);
+    else no_print = no_pf = (hook & 1);
+    if ((hook & 2)
+        && globTab[index].ops == &dataBlockSym
+        && globTab[index].value.db->ops == &functionOps) {
+      /* if alternate after_error function present, make it the idler */
+      y_idler_function = (Function *)Ref(globTab[index].value.db);
+      no_reset = ((hook & 4) != 0);
+    }
+  }
   if (!no_print) YputsErr(yErrorMsg);
 
   if (recursing) {
     func= 0;
-    YputsErr("WARNING aborting on recursive calls to YError");
+    if (!no_print) YputsErr("WARNING aborting on recursive calls to YError");
   } else if (ym_state&Y_SUSPENDED) {
     func= 0;
-    if (y_read_prompt)
-      YputsErr("WARNING aborting on YError during keyboard read()");
-    else if (ym_suspc)
-      YputsErr("WARNING aborting on YError during suspend");
-    else
-      YputsErr("WARNING aborting on YError after mouse() pause() or wait=1");
+    if (y_read_prompt) {
+      if (!no_print) YputsErr("WARNING aborting on YError"
+                              " during keyboard read()");
+    } else if (ym_suspc) {
+      if (!no_print) YputsErr("WARNING aborting on YError during suspend");
+    } else {
+      if (!no_print) YputsErr("WARNING aborting on YError"
+                              " after mouse() pause() or wait=1");
+    }
   }
   ym_suspc = 0;
   if (yg_blocking==3 || yg_blocking==4) yg_blocking = 0;
@@ -1083,8 +1550,12 @@ YError(const char *msg)
 
     if (y_idler_function) {
       /* special after_error function will get control */
-      ResetStack(1);
-      yr_reset();
+      if (no_reset && yDebugLevel<=2 && !y_read_prompt) {
+        if (yDebugLevel>1) ResetStack(0);
+      } else {
+        ResetStack(1);
+        yr_reset();
+      }
       yg_got_expose();
       p_clr_alarm(0, 0);
     } else if (!yBatchMode && !pcUp && (!yAutoDebug || yDebugLevel>1)) {
@@ -1103,8 +1574,8 @@ YError(const char *msg)
     if (recursing<5) {
       ResetStack(y_read_prompt!=0);
     } else {
-      YputsErr("****SEVERE**** YError unable to reset stack -- "
-               "probably lost variables");
+      if (!no_pf) YputsErr("****SEVERE**** YError unable to reset stack -- "
+                           "probably lost variables");
       sp= spBottom;
     }
     ym_state &= ~Y_SUSPENDED;
@@ -1114,21 +1585,24 @@ YError(const char *msg)
   }
 
   if (ym_state&Y_QUITTING) {
-    YputsErr("****TERMINATING**** on error after main loop exit");
+    if (!no_pf)
+      YputsErr("****TERMINATING**** on error after main loop exit");
     if (!no_abort) p_abort();
     return;
   }
 
   if ((yBatchMode && !y_idler_function) || !strncmp(msg,"(FATAL)",7)) {
-    if (yBatchMode) YputsErr("yorick: quitting on error in batch mode");
-    else YputsErr("yorick: quitting on fatal error");
+    if (!no_pf) {
+      if (yBatchMode) YputsErr("yorick: quitting on error in batch mode");
+      else YputsErr("yorick: quitting on fatal error");
+    }
     ym_fatal = yBatchMode? 1 : 2;
     ResetStack(0);
     ym_state|= Y_QUITTING;
   }
 
   /* set catch_message variable for after_error function */
-  if (y_idler_function) yset_catchmsg(tmsg);
+  if (y_idler_function) yset_catchmsg(yErrorMsg);
 
   /* Go back to the main loop.  */
   inYError= 0;
@@ -1363,13 +1837,16 @@ void Y_catch(int nArgs)
 void Y_set_idler(int nArgs)
 {
   Function *f;
-  if (nArgs>1)
-    YError("set_idler function takes zero or one arguments");
+  if (nArgs>1) {
+    /* second argument is hack to add error handling features */
+    if (nArgs==2 && sp[-1].ops) yerror_flags = YGetInteger(sp);
+    else YError("set_idler function takes zero or one arguments");
+  }
 
   if (nArgs>0 && YNotNil(sp-nArgs+1)) {
     f = (Function *)sp[1-nArgs].value.db;
     if (sp[1-nArgs].ops!=&dataBlockSym || f->ops!=&functionOps)
-      YError("expecting function as argument");
+      YError("set_idler expecting function as argument");
     y_idler_function = Ref(f);
 
   } else if (y_idler_function) {
