@@ -1,11 +1,12 @@
 /*
-    YDATA.C
-    Implement functions for Yorick-specific types of data.
-
-    $Id: ydata.c,v 1.1 1993/08/27 18:32:09 munro Exp $
+ * $Id: ydata.c,v 1.4 2010/04/16 05:23:43 dhmunro Exp $
+ * Implement functions for Yorick-specific types of data.
+ */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
  */
-/*    Copyright (c) 1994.  The Regents of the University of California.
-                    All rights reserved.  */
 
 #include "bcast.h"
 #include "defmem.h"
@@ -18,6 +19,7 @@
 extern Array *GrowArray(Array *array, long extra);
 
 extern BuiltIn Y_yorick_stats, Y_symbol_def, Y_symbol_set;
+extern BuiltIn Y_symbol_names, Y_symbol_exists, Y_errs2caller;
 
 /* Required for FetchLValue, StoreLValue */
 extern void ReadGather(void *dst, void *srcM, long srcD, StructDef *base,
@@ -64,11 +66,13 @@ Function *NewFunction(Symbol *consts, long nConsts, int nPos, int nKey,
   func->ops= &functionOps;
   func->constantTable= consts;
   func->nConstants= nConsts;
-  func->nReq= frameSize+maxStackDepth+2;
+  /* guarantee 8 stack slots available for builtins, plus 2 for good luck */
+  func->nReq= frameSize+maxStackDepth+10;
   func->nPos= nPos;
   func->nKey= nKey;
   func->nLocal= nLocal;
   func->hasPosList= hasPL;
+  func->errup = 0;
   codeSize-= frameSize-1;
   /* YpFunc puts the frame variables (parameters and locals) at the end
      of the code, switch them to the beginning now.  */
@@ -94,6 +98,20 @@ void FreeFunction(void *v)  /* ******* Use Unref(func) ******* */
   p_free(func);
 }
 
+void
+Y_errs2caller(int argc)
+{
+  Function *f;
+  int i;
+  for (i=argc-1 ; i>=0 ; i--) {
+    if (sp[-i].ops == &referenceSym) ReplaceRef(sp-i);
+    if (sp[-i].ops!=&dataBlockSym || sp[-i].value.db->ops!=&functionOps)
+      YError("errs2caller accepts only function arguments");
+    f = (Function *)sp[-i].value.db;
+    f->errup = 1;
+  }
+}
+
 /* Set up a block allocator which grabs space for 64 range objects
    at a time.  Since Range contains an ops pointer, the alignment
    of a Range must be at least as strict as a void*.  */
@@ -505,6 +523,19 @@ int CalledAsSubroutine(void)
   return pc->Action==&DropTop;
 }
 
+Operand *
+yarg_op(int iarg, Operand *op)
+{
+  if (iarg>=0) {
+    Symbol *s = sp - iarg;
+    if (s->ops) s->ops->FormOperand(s, op);
+    else op->ops=0, op=0;
+  } else {
+    op = 0;
+  }
+  return op;
+}
+
 long YGetInteger(Symbol *s)
 {
   Operand op;
@@ -518,6 +549,12 @@ long YGetInteger(Symbol *s)
   return *(long *)op.value;
 }
 
+long
+yarg_sl(int iarg)
+{
+  return (iarg>=0)? YGetInteger(sp-iarg) : 0;
+}
+
 double YGetReal(Symbol *s)
 {
   Operand op;
@@ -531,6 +568,12 @@ double YGetReal(Symbol *s)
   return *(double *)op.value;
 }
 
+double
+yarg_sd(int iarg)
+{
+  return (iarg>=0)? YGetReal(sp-iarg) : 0;
+}
+
 char *YGetString(Symbol *s)
 {
   Operand op;
@@ -541,6 +584,12 @@ char *YGetString(Symbol *s)
   return *(char **)op.value;
 }
 
+char *
+yarg_sq(int iarg)
+{
+  return (iarg>=0)? YGetString(sp-iarg) : 0;
+}
+
 int YNotNil(Symbol *s)
 {
   if (!s) return 0;  /* for use with YGetKeywords */
@@ -571,6 +620,25 @@ Symbol *YGetKeywords(Symbol *stack, int nArgs, char **keyNames,
   return s0? s0 : stack+1;
 }
 
+int
+yarg_keys(int iarg, char **knames, Symbol **ksymbols)
+{
+  char *key;
+  int i;
+  while (iarg > 0) {
+    if (sp[-iarg].ops) break;
+    key = globalTable.names[sp[-(iarg--)].index];
+    for (i=0 ; knames[i] ; i++) {
+      if (ksymbols[i]) continue;
+      if (strcmp(key, knames[i]) == 0) break;
+    }
+    if (!knames[i])
+      YError("unrecognized or duplicate keyword");
+    ksymbols[i] = sp - (iarg--);
+  }
+  return iarg;
+}
+
 IOStream *YGetFile(Symbol *stack)
 {
   IOStream *file;
@@ -583,6 +651,12 @@ IOStream *YGetFile(Symbol *stack)
   return file;
 }
 
+IOStream *
+yarg_file(int iarg)
+{
+  return (iarg>=0)? YGetFile(sp-iarg) : 0;
+}
+
 /*--------------------------------------------------------------------------*/
 
 /* Retrieve array arguments for foreign code wrappers,
@@ -601,6 +675,12 @@ char *YGet_C(Symbol *s, int nilOK, Dimension **dims)
   return op.value;
 }
 
+char *
+yarg_c(int iarg, Dimension **dims)
+{
+  return (iarg>=0)? YGet_C(sp-iarg, 0, dims) : 0;
+}
+
 short *YGet_S(Symbol *s, int nilOK, Dimension **dims)
 {
   Operand op;
@@ -612,6 +692,12 @@ short *YGet_S(Symbol *s, int nilOK, Dimension **dims)
   return op.value;
 }
 
+short *
+yarg_s(int iarg, Dimension **dims)
+{
+  return (iarg>=0)? YGet_S(sp-iarg, 0, dims) : 0;
+}
+
 int *YGet_I(Symbol *s, int nilOK, Dimension **dims)
 {
   Operand op;
@@ -627,6 +713,12 @@ int *YGet_I(Symbol *s, int nilOK, Dimension **dims)
   return op.value;
 }
 
+int *
+yarg_i(int iarg, Dimension **dims)
+{
+  return (iarg>=0)? YGet_I(sp-iarg, 0, dims) : 0;
+}
+
 long *YGet_L(Symbol *s, int nilOK, Dimension **dims)
 {
   Operand op;
@@ -642,6 +734,12 @@ long *YGet_L(Symbol *s, int nilOK, Dimension **dims)
   return op.value;
 }
 
+long *
+yarg_l(int iarg, Dimension **dims)
+{
+  return (iarg>=0)? YGet_L(sp-iarg, 0, dims) : 0;
+}
+
 float *YGet_F(Symbol *s, int nilOK, Dimension **dims)
 {
   Operand op;
@@ -653,6 +751,12 @@ float *YGet_F(Symbol *s, int nilOK, Dimension **dims)
   return op.value;
 }
 
+float *
+yarg_f(int iarg, Dimension **dims)
+{
+  return (iarg>=0)? YGet_F(sp-iarg, 0, dims) : 0;
+}
+
 double *YGet_D(Symbol *s, int nilOK, Dimension **dims)
 {
   Operand op;
@@ -668,6 +772,12 @@ double *YGet_D(Symbol *s, int nilOK, Dimension **dims)
   return op.value;
 }
 
+double *
+yarg_d(int iarg, Dimension **dims)
+{
+  return (iarg>=0)? YGet_D(sp-iarg, 0, dims) : 0;
+}
+
 double *YGet_Z(Symbol *s, int nilOK, Dimension **dims)
 {
   Operand op;
@@ -679,6 +789,12 @@ double *YGet_Z(Symbol *s, int nilOK, Dimension **dims)
   return op.value;
 }
 
+double *
+yarg_z(int iarg, Dimension **dims)
+{
+  return (iarg>=0)? YGet_Z(sp-iarg, 0, dims) : 0;
+}
+
 char **YGet_Q(Symbol *s, int nilOK, Dimension **dims)
 {
   Operand op;
@@ -690,6 +806,12 @@ char **YGet_Q(Symbol *s, int nilOK, Dimension **dims)
   return op.value;
 }
 
+char **
+yarg_q(int iarg, Dimension **dims)
+{
+  return (iarg>=0)? YGet_Q(sp-iarg, 0, dims) : 0;
+}
+
 void **YGet_P(Symbol *s, int nilOK, Dimension **dims)
 {
   Operand op;
@@ -701,6 +823,12 @@ void **YGet_P(Symbol *s, int nilOK, Dimension **dims)
   return op.value;
 }
 
+void **
+yarg_p(int iarg, Dimension **dims)
+{
+  return (iarg>=0)? YGet_P(sp-iarg, 0, dims) : 0;
+}
+
 int YGet_dims(const Dimension *dims, long *dlist, int maxDims)
 {
   int i, n= CountDims(dims);
@@ -753,7 +881,7 @@ void Y_yorick_stats(int nArgs)
   tmpDims= NewDimension(14L, 1L, (Dimension *)0);
   result= PushDataBlock(NewArray(&longStruct, tmpDims));
   result->value.l[0]= globalTable.nItems;
-  result->value.l[1]= yStackSize;
+  result->value.l[1]=  yStackSize;
   result->value.l[2]= p_nallocs-p_nfrees;
   result->value.l[3]= p_asmall;
   result->value.l[4]= ExtraNilRefs();
@@ -810,4 +938,151 @@ void Y_symbol_set(int nArgs)
   glob->ops= sp->ops;
 }
 
+void Y_symbol_exists(int argc)
+{
+  if (argc != 1) YError("symbol_exists takes exactly one argument");
+  PushIntValue(HashFind(&globalTable, YGetString(sp), 0L));
+}
+
+#define GET_ARRAY       1
+#define GET_STRUCT      2
+#define GET_RANGE       4
+#define GET_VOID        8
+#define GET_FUNCTION   16
+#define GET_BUILTIN    32
+#define GET_STRUCTDEF  64
+#define GET_STREAM    128
+#define GET_OPAQUE    256
+#define GET_LIST      512
+#define GET_AUTOLOAD 1024
+
+void Y_symbol_names(int argc)
+{
+  extern Operations listOps;
+  long i, nitems, number;
+  char **ret;
+  int match[T_OPAQUE+1];
+  int type, flags, pass;
+  int omit_array, omit_list, omit_autoload, omit_opaque;
+  Dimension *dims = tmpDims;
+
+  tmpDims = (Dimension *)0;
+  if (dims != (Dimension *)0) FreeDimension(dims);
+  if (argc != 1) YError("symbol_list takes exactly one argument");
+  if (YNotNil(sp)) {
+    flags = YGetInteger(sp);
+  } else {
+    flags = (GET_ARRAY | GET_STRUCT | GET_RANGE | GET_FUNCTION | GET_BUILTIN |
+             GET_STRUCTDEF | GET_STREAM | GET_OPAQUE);
+  }
+  nitems = globalTable.nItems;
+  if (nitems <= 0) {
+    /* No symbols defined. */
+    PushDataBlock(RefNC(&nilDB));
+    return;
+  }
+  if (flags == -1) {
+    /* Return names of all symbols ever defined. */
+    tmpDims = NewDimension(nitems, 1L, (Dimension *)0);
+    ret = ((Array *)PushDataBlock(NewArray(&stringStruct, tmpDims)))->value.q;
+    for (i = 0; i < nitems; ++i) {
+      ret[i] = p_strcpy(globalTable.names[i]);
+    }
+    return;
+  }
+  omit_array = ((flags & GET_ARRAY) == 0);
+  omit_list = ((flags & GET_LIST) == 0);
+  omit_autoload = ((flags & GET_AUTOLOAD) == 0);
+  omit_opaque = ((flags & GET_OPAQUE) == 0);
+  if ((flags & (GET_LIST | GET_AUTOLOAD)) != 0) {
+    flags |= GET_OPAQUE;
+  }
+  for (i = 0; i <= T_OPAQUE; ++i) {
+    match[i] = 0;
+  }
+  match[T_CHAR]      = ((flags & GET_ARRAY) != 0);
+  match[T_SHORT]     = ((flags & GET_ARRAY) != 0);
+  match[T_INT]       = ((flags & GET_ARRAY) != 0);
+  match[T_LONG]      = ((flags & GET_ARRAY) != 0);
+  match[T_FLOAT]     = ((flags & GET_ARRAY) != 0);
+  match[T_DOUBLE]    = ((flags & GET_ARRAY) != 0);
+  match[T_COMPLEX]   = ((flags & GET_ARRAY) != 0);
+  match[T_STRING]    = ((flags & GET_ARRAY) != 0);
+  match[T_POINTER]   = ((flags & GET_ARRAY) != 0);
+  match[T_STRUCT]    = ((flags & GET_STRUCT) != 0);
+  match[T_RANGE]     = ((flags & GET_RANGE) != 0);
+#ifdef GET_LVALUE
+  match[T_LVALUE]    = ((flags & GET_LVALUE) != 0);
+#endif
+  match[T_VOID]      = ((flags & GET_VOID) != 0);
+  match[T_FUNCTION]  = ((flags & GET_FUNCTION) != 0);
+  match[T_BUILTIN]   = ((flags & GET_BUILTIN) != 0);
+  match[T_STRUCTDEF] = ((flags & GET_STRUCTDEF) != 0);
+  match[T_STREAM]    = ((flags & GET_STREAM) != 0);
+  match[T_OPAQUE]    = ((flags & GET_OPAQUE) != 0);
+
+  /* Counter number of matching symbols. */
+  ret = NULL; /* avoids compiler warning */
+  number = 0;
+  for (pass = 0; pass <= 1; ++pass) {
+    if (pass) {
+      if (number <= 0) {
+	/* No matching symbols found. */
+	PushDataBlock(RefNC(&nilDB));
+	return;
+      }
+      tmpDims = NewDimension(number, 1L, (Dimension *)0);
+      ret = ((Array *)PushDataBlock(NewArray(&stringStruct, tmpDims)))->value.q;
+    }
+    for (i=0 ; i<nitems ; ++i) {
+      OpTable *sym_ops = globTab[i].ops;
+      if (sym_ops == &dataBlockSym) {
+	Operations *ops = globTab[i].value.db->ops;
+	type = ops->typeID;
+	if ((unsigned int)type > T_OPAQUE || ! match[type]) {
+	  continue;
+	}
+	if (type == T_OPAQUE) {
+	  if (ops == &listOps) {
+	    if (omit_list) {
+	      continue;
+	    }
+	  } else if (ops == &auto_ops) {
+	    if (omit_autoload) {
+	      continue;
+	    }
+	  } else {
+	    if (omit_opaque) {
+	      continue;
+	    }
+	  }
+	}
+      } else if (sym_ops == &longScalar ||
+		 sym_ops == &intScalar ||
+		 sym_ops == &doubleScalar) {
+	if (omit_array) {
+	  continue;
+	}
+      }
+      if (pass) {
+	*ret++ = p_strcpy(globalTable.names[i]);
+      } else {
+	++number;
+      }
+    }
+  }
+}
+
+#undef GET_ARRAY
+#undef GET_STRUCT
+#undef GET_RANGE
+#undef GET_VOID
+#undef GET_FUNCTION
+#undef GET_BUILTIN
+#undef GET_STRUCTDEF
+#undef GET_STREAM
+#undef GET_OPAQUE
+#undef GET_LIST
+#undef GET_AUTOLOAD
+
 /*--------------------------------------------------------------------------*/
