@@ -1,11 +1,12 @@
 /*
-    YINPUT.C
-    Implement Yorick program text reader.
-
-    $Id: yinput.c,v 1.1 1993/08/27 18:32:09 munro Exp $
+ * $Id: yinput.c,v 1.4 2010/02/15 05:17:57 dhmunro Exp $
+ * Implement Yorick program text reader.
+ */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
  */
-/*    Copyright (c) 1994.  The Regents of the University of California.
-                    All rights reserved.  */
 
 #include "yio.h"
 #include "parse.h"
@@ -22,15 +23,6 @@ extern void YError(const char *msg);
 extern int yDebugLevel;        /* declared in ydata.h */
 extern HashTable globalTable;  /* declared in ydata.h */
 
-extern long ypBeginLine; /* defined in this file -- line number where
-                            most recent parse began for YError */
-
-extern int ypSkipIncludes; /* defined in this file -- flag set when
-                              scanning for a variable definition */
-
-extern long YpStandby(void);    /* used by ScanForFunc in yorick.y */
-extern long ScanForFunc(const char *fname, int notExtern);
-
 extern char *MakeErrorLine(long lineNumber, const char *filename);
 
 extern long YpLineNumber(void);
@@ -166,14 +158,31 @@ static void AddPrefix(char *prefix)
 /* state for YpError initialized in YpPushInclude */
 static long prevErrLine;
 
-static p_file *PushInclude(const char *filename, int clear);
+static p_file *PushInclude(const char *filename, int fullparse);
 
 p_file *YpPushInclude(const char *filename)
 {
   return PushInclude(filename, 1);
 }
 
-static p_file *PushInclude(const char *filename, int clear)
+static p_file *default_on_include(const char *filename, int fullparse);
+static yon_include_cb *open_include = &default_on_include;
+/* ARGSUSED */
+static p_file *
+default_on_include(const char *filename, int fullparse)
+{
+  return p_fopen(filename, "r");
+}
+
+yon_include_cb *
+ycall_on_include(yon_include_cb *on_include)
+{
+  yon_include_cb *old = open_include;
+  open_include = on_include? on_include : &default_on_include;
+  return old;
+}
+
+static p_file *PushInclude(const char *filename, int fullparse)
 {
   p_file *file= 0;
   char *name= 0;
@@ -181,7 +190,7 @@ static p_file *PushInclude(const char *filename, int clear)
 
   if (YIsAbsolute(filename)) {
     /* absolute pathname doesn't need any prefix */
-    file= p_fopen(filename, "r");
+    file= open_include(filename, fullparse);
     if (!file) return 0;
     name= p_strcpy(filename);
 
@@ -200,7 +209,7 @@ static p_file *PushInclude(const char *filename, int clear)
         name= YExpandName(filename);
         if (!YIsAbsolute(name)) break;
       }
-      file= p_fopen(name, "r");
+      file= open_include(name, fullparse);
       if (file) break;
       p_free(name);
     }
@@ -213,15 +222,32 @@ static p_file *PushInclude(const char *filename, int clear)
     maxYpIncludes= newSize;
   }
 
-  if (clear) ClearSourceList(name);
+  if (fullparse) ClearSourceList(name);
 
   ypIncludes[nYpIncludes].file= file;
   ypIncludes[nYpIncludes].filename= name;
-  ypIncludes[nYpIncludes++].lastLineRead= 0;
+  ypIncludes[nYpIncludes].lastLineRead= 0;
+  ypIncludes[nYpIncludes++].index = -1;
   prevErrLine= -1;
   return file;
 }
 
+void
+y_push_include(p_file *file, const char *filename)
+{
+  if (nYpIncludes >= maxYpIncludes) {
+    int newSize = maxYpIncludes + 4;
+    ypIncludes = p_realloc(ypIncludes, sizeof(IncludeFile)*newSize);
+    maxYpIncludes = newSize;
+  }
+
+  ypIncludes[nYpIncludes].file = file;
+  ypIncludes[nYpIncludes].filename = p_strcpy(filename);
+  ypIncludes[nYpIncludes].lastLineRead = 0;
+  ypIncludes[nYpIncludes++].index = -1;
+  prevErrLine= -1;
+}
+
 static int need_endif= 0;
 
 void YpClearIncludes(void)
@@ -230,9 +256,6 @@ void YpClearIncludes(void)
   if (nYpIncludes>0) {
     do {
       nYpIncludes--;
-      /* Note <stdio.h> might not contain any prototype for fclose, or it
-         might contain the correct ANSI prototype, or it might contain
-         a K+R style function declaration...  */
       if (ypIncludes[nYpIncludes].file)
         p_fclose(ypIncludes[nYpIncludes].file);
       p_free(ypIncludes[nYpIncludes].filename);
@@ -390,13 +413,14 @@ y_remove_line(void)
   if (y_lhead) {
     y_line_t *yline = y_lhead;
     y_line_t *next = yline->next;
-    char *line = y_lhead->line;
+    char *line = yline->line;
     y_lhead = next;
     if (next)
       yline->next = 0;
     else
       y_ltail = 0;
     yline->line = 0;
+    p_free(yline);
     if (line) p_free(line);
   }
 }
@@ -410,8 +434,6 @@ y_on_stdin(char *input_line)
     if (input_line)
       while (input_line[0]==' ' || input_line[0]=='\t') input_line++;
     if (flag && input_line && input_line[0]) {
-      extern void ResetStack(int hard);
-      extern void yr_reset(void);
       extern void yg_got_expose(void);
       yp_continue = 0;
       if (p_signalling) p_abort();
@@ -575,11 +597,12 @@ YpNextLine(int context)
   /* get the current include file */
   if (nYpIncludes==0) {
     file = 0;
-  } else do {
+  } else for (;;) {
     if ((file=ypIncludes[nYpIncludes-1].file)) break;
     need_endif = 0;
     p_free(ypIncludes[nYpIncludes-1].filename);
-  } while (--nYpIncludes);
+    if (!(--nYpIncludes)) break;
+  }
   if (!file && nYpInputs) do file = YpPop(); while (!file && nYpInputs);
 
   /* quit if input is not from include file.  */
@@ -631,6 +654,7 @@ char *MakeErrorLine(long lineNumber, const char *filename)
 }
 
 extern int yBatchMode;  /* may be set with -batch, see std0.c */
+PLUG_API int yerror_flags;  /* see task.c */
 
 void YpError(char *msg)
 {
@@ -638,19 +662,22 @@ void YpError(char *msg)
   ypErrors++;
   strcpy(pErrorMsg, "SYNTAX: ");
   strncat(pErrorMsg, msg, 110);
-  YputsErr(pErrorMsg);
-  if (nYpIncludes) {
-    long lineNumber= ypIncludes[nYpIncludes-1].lastLineRead;
-    if (lineNumber!=prevErrLine) {
-      char *filename= ypIncludes[nYpIncludes-1].filename;
-      YputsErr(MakeErrorLine(lineNumber, filename));
-      prevErrLine= lineNumber;
+  if (!(yerror_flags&1)) {
+    YputsErr(pErrorMsg);
+    if (nYpIncludes) {
+      long lineNumber= ypIncludes[nYpIncludes-1].lastLineRead;
+      if (lineNumber!=prevErrLine) {
+        char *filename= ypIncludes[nYpIncludes-1].filename;
+        YputsErr(MakeErrorLine(lineNumber, filename));
+        prevErrLine= lineNumber;
+      }
     }
   }
-  if (yBatchMode || ypErrors>=ypMaxErrors) {
+  if (yBatchMode || ypErrors>=ypMaxErrors || (yerror_flags&1)) {
     ypErrors= 0;
     if (ypMaxErrors<1) ypMaxErrors= 1;
-    YError("****ABORTING PARSE**** too many errors");
+    if (yerror_flags&1) YError(pErrorMsg);
+    else YError("****ABORTING PARSE**** too many errors");
   }
 }
 
@@ -691,8 +718,10 @@ long ReopenSource(long index, int notExtern)
 
   position= ScanForFunc(globalTable.names[index], notExtern);
   if (position<0 || p_fseek(file, position)) {
-    p_fclose(file);
-    ypIncludes[nYpIncludes-1].file= 0;
+    if (ypIncludes[nYpIncludes-1].file) {
+      p_fclose(file);
+      ypIncludes[nYpIncludes-1].file= 0;
+    }
     if (position<0) return -3;  /* func no longer in source file */
     else return -4;             /* seek error */
   }
